# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301

"""
EMA.

Configuration keys:

[ema]

scp-src = [user@server:[port]]/path/to/file.mrc
"""

from gluish.benchmark import timed
from gluish.esindex import CopyToIndex
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.configuration import Config
from siskin.task import DefaultTask
import datetime
import luigi

config = Config.instance()

class EMATask(DefaultTask):
    TAG = '009'

    def closest(self):
        return datetime.date(2014, 1, 1)

class EMASync(EMATask):
    date = ClosestDateParameter(default=datetime.date.today())

    @timed
    def run(self):
        output = shellout("scp {origin} {output}", origin=config.get('ema', 'scp-src'))
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='mrc'))

class EMAJson(EMATask):
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return EMASync(date=self.date)

    @timed
    def run(self):
        output = shellout("marctojson -m date={date} {input} > {output}",
                          input=self.input().path, date=self.closest())
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj'))

class EMAIndex(EMATask, CopyToIndex):
    date = ClosestDateParameter(default=datetime.date.today())

    index = 'ema'
    doc_type = 'title'
    purge_existing_index = True

    mapping = {'title': {'date_detection': False,
                          '_id': {'path': 'content.001'},
                          '_all': {'enabled': True,
                                   'term_vector': 'with_positions_offsets',
                                   'store': True}}}
    def update_id(self):
        """ This id will be a unique identifier for this indexing task."""
        return self.effective_task_id()

    def requires(self):
        return EMAJson(date=self.date)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301

"""
KSD.

Configuration keys:

[ksd]

scp-src = [user@server:[port]]/path/to/*.mrc

"""

from gluish.benchmark import timed
from gluish.esindex import CopyToIndex
from gluish.parameter import ClosestDateParameter
from gluish.path import iterfiles
from gluish.utils import shellout
from siskin.configuration import Config
from siskin.task import DefaultTask
import datetime
import luigi
import tempfile

config = Config.instance()

class KSDTask(DefaultTask):
    TAG = '019'

    def closest(self):
        """ Single shipment. """
        return datetime.date(2014, 1, 1)

class KSDImport(KSDTask):
    """ Add the file on mdma (or wherever) and adjust the latest date. """
    date = ClosestDateParameter(default=datetime.date.today())
    tag = luigi.Parameter('haab')

    @timed
    def run(self):
        # gather files
        stopover = tempfile.mkdtemp(prefix='siskin-')
        shellout("scp {origin} {output}", origin=config.get('ksd', 'scp-src'), output=stopover)
        # combine files
        _, combined = tempfile.mkstemp(prefix='siskin-')
        for path in sorted(iterfiles(stopover), reverse=True):
            shellout("cat {input} >> {output}", input=path, output=combined)
        # clean dups
        output = shellout("marcuniq {input} > {output}", input=combined)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='mrc'))

class KSDJson(KSDTask):
    """ Convert to JSON. """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return KSDImport(date=self.date)

    @timed
    def run(self):
        output = shellout("marctojson -m date={date} {input} > {output}",
                        input=self.input().path, date=self.closest())
        luigi.File(output).move(self.output().path)

    def output(self):
        """ Use monthly files. """
        return luigi.LocalTarget(path=self.path(ext='ldj'))

class KSDIndex(KSDTask, CopyToIndex):
    date = ClosestDateParameter(default=datetime.date.today())

    index = 'ksd'
    doc_type = 'title'

    mapping = {'title': {'date_detection': False,
                      '_id': {'path': 'content.001'},
                      '_all': {'enabled': True,
                               'term_vector': 'with_positions_offsets',
                               'store': True}}}

    def update_id(self):
        """ This id will be a unique identifier for this indexing task."""
        return self.effective_task_id()

    def requires(self):
        return KSDJson(date=self.date)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301

"""
Elsevier.

Configuration keys:

[elsevier]

scp-src = [user@server:[port]]/path/to/file.bin
"""

from gluish.benchmark import timed
from gluish.common import Executable
from gluish.esindex import CopyToIndex
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.configuration import Config
from siskin.task import DefaultTask
import datetime
import luigi

config = Config.instance()

class ElsevierTask(DefaultTask):
    TAG = '016'

    def closest(self):
        return datetime.date(2014, 1, 1)

class ElsevierJson(ElsevierTask):
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return [Executable(name='marcuniq'), Executable(name='marctojson')]

    @timed
    def run(self):
        output = shellout("scp {origin} {output}", origin=config.get('elsevier', 'scp-src'))
        output = shellout("marcuniq {input} > {output}", input=output)
        output = shellout("marctojson -m date={date} {input} > {output}",
                          date=self.closest(), input=output)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj'))

class ElsevierIndex(ElsevierTask, CopyToIndex):
    date = ClosestDateParameter(default=datetime.date.today())

    index = 'elsevier'
    doc_type = 'title'
    purge_existing_index = True

    mapping = {'title': {'date_detection': False,
                          '_id': {'path': 'content.001'},
                          '_all': {'enabled': True,
                                   'term_vector': 'with_positions_offsets',
                                   'store': True}}}

    def update_id(self):
        """ This id will be a unique identifier for this indexing task."""
        return self.effective_task_id()

    def requires(self):
        return ElsevierJson(date=self.date)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101

"""
GBV related tasks.

Configutation keys:

[gbv]

scp-src = [user@server:[port]]/path/to/file/{tag}/*mrc

Requirements:

    * marcuniq, marctojson
    * elasticsearch
"""

from gluish.benchmark import timed
from gluish.esindex import CopyToIndex
from gluish.parameter import ClosestDateParameter
from gluish.path import iterfiles
from gluish.utils import shellout
from siskin.configuration import Config
from siskin.task import DefaultTask
import datetime
import luigi
import tempfile

config = Config.instance()

class GBVTask(DefaultTask):
    """ GBV base task. """
    TAG = '021'

    def closest(self):
        """ Singular event. """
        return datetime.date(2014, 1, 1)

class GBVImport(GBVTask):
    """ Copy and merge files from a single tag on the fly. Tag is the name of
    the subdirectory, like haab, hfm or sbb.
    """
    date = ClosestDateParameter(default=datetime.date.today())
    tag = luigi.Parameter(default='haab')

    @timed
    def run(self):
        stopover = tempfile.mkdtemp(prefix='siskin-')
        origin = config.get('gbv', 'scp-src').format(tag=self.tag)
        shellout("scp {origin} {output}", origin=origin, output=stopover)

        _, combined = tempfile.mkstemp(prefix='siskin-')
        for path in iterfiles(stopover):
            shellout("cat {input} >> {output}", input=path, output=combined)
        luigi.File(combined).move(self.output().fn)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='mrc'))

class GBVCombine(GBVTask):
    """ Combine different files from different tags together. """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        for tag in ('haab', 'hfm', 'sbb'):
            yield GBVImport(date=self.date, tag=tag)

    @timed
    def run(self):
        _, combined = tempfile.mkstemp(prefix='siskin-')
        for target in self.input():
            shellout("cat {input} >> {output}", input=target.path,
                     output=combined)
        output = shellout("marcuniq {input} > {output}", input=combined)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='mrc'))

class GBVJson(GBVTask):
    """ Convert to Json. """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return GBVCombine(date=self.date)

    @timed
    def run(self):
        output = shellout("marctojson -m date={date} {input} > {output}",
                          input=self.input().path, date=self.closest())
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj'))

class GBVIndex(GBVTask, CopyToIndex):
    """ Upload to ES. """
    date = ClosestDateParameter(default=datetime.date.today())

    index = 'gbv'
    doc_type = 'title'
    purge_existing_index = True

    mapping = {'title': {'date_detection': False,
                          '_id': {'path': 'content.001'},
                          '_all': {'enabled': True,
                                   'term_vector': 'with_positions_offsets',
                                   'store': True}}}

    def update_id(self):
        """ This id will be a unique identifier for this indexing task."""
        return self.effective_task_id()

    def requires(self):
        return GBVJson(date=self.date)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301

"""
HMT related workflows.

Data reconciliation, with target artefacts and precision reports.
"""

from __future__ import print_function
from gluish.format import TSV
from gluish.intervals import weekly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.sources.naxos import NaxosJson
from siskin.task import DefaultTask
from siskin.utils import ElasticsearchMixin
import datetime
import itertools
import json
import luigi


class HMTTask(DefaultTask):
    TAG = 'hmt'
    NULL = '<NULL>'

    def closest(self):
        return weekly(date=self.date)

class HMTNaxosPersonsAndDates(HMTTask):
    """
    Dump all people and their bio data from 100 and 700 fields.
    """
    date = ClosestDateParameter(default=datetime.date.today())
    encoding = luigi.Parameter(default='utf-8', significant=False)

    def requires(self):
        return NaxosJson(date=self.date)

    def run(self):
        with self.input().open() as handle:
            with self.output().open('w') as output:
                for _, line in enumerate(handle):
                    c = json.loads(line).get('content')
                    id = c.get('001', self.NULL)
                    for field in itertools.chain(c.get('100', []), c.get('700', [])):
                        output.write_tsv(id, field.get('a').encode(self.encoding), field.get('d', self.NULL))

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)

class HMTNaxosUniqPersonsAndDates(HMTTask):
    """ Return the unique authors. """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return HMTNaxosPersonsAndDates(date=self.date)

    def run(self):
        output = shellout("cut -f 2-3 < {input} | LANG=C sort -u > {output}",
                          input=self.input().path)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)

class HMTNaxosCompletePersonsAndDates(HMTTask):
    """
    Only names with dates.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return HMTNaxosUniqPersonsAndDates(date=self.date)

    def run(self):
        output = shellout("""LANG=C grep -v "{null}" {input} > {output}""",
                          null=self.NULL, input=self.input().path)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)

class HMTNaxosIncompleteAuthorsAndDates(HMTTask):
    """
    Only names without dates.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return HMTNaxosUniqPersonsAndDates(date=self.date)

    def run(self):
        output = shellout("""LANG=C grep "{null}" {input} > {output}""",
                          null=self.NULL, input=self.input().path)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301

"""
MOR.

Configuration keys:

[mor]

ftp-host = host.name
ftp-username = username
ftp-password = password
ftp-base = .
ftp-pattern = *

"""

from gluish.benchmark import timed
from gluish.common import FTPMirror
from gluish.esindex import CopyToIndex
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.configuration import Config
from siskin.task import DefaultTask
import datetime
import luigi
import tempfile

config = Config.instance()

class MORTask(DefaultTask):
    TAG = '006'

    def closest(self):
        return datetime.date(2014, 1, 1)

class MORJson(MORTask):
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return FTPMirror(host=config.get('mor', 'ftp-host'),
                         username=config.get('mor', 'ftp-username'),
                         password=config.get('mor', 'ftp-password'),
                         base=config.get('mor', 'ftp-base'),
                         pattern=config.get('mor', 'ftp-pattern'))

    @timed
    def run(self):
        _, combined = tempfile.mkstemp(prefix='tasktree-')
        with self.input().open() as handle:
            for row in handle.iter_tsv(cols=('path',)):
                if row.path.endswith('.xml'):
                    output = shellout("""yaz-marcdump -i marcxml -o marc
                                      {input} >> {output}""", input=row.path,
                                      output=combined)
        output = shellout("""marcuniq {input} > {output}""", input=combined)
        output = shellout("""marctojson -m date={date} {input} > {output}""",
                          date=self.closest(), input=output)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='json'))

class MORIndex(MORTask, CopyToIndex):
    date = ClosestDateParameter(default=datetime.date.today())

    index = 'mor'
    doc_type = 'title'
    purge_existing_index = True

    mapping = {'title': {'date_detection': False,
                          '_id': {'path': 'content.001'},
                          '_all': {'enabled': True,
                                   'term_vector': 'with_positions_offsets',
                                   'store': True}}}

    def update_id(self):
        """ This id will be a unique identifier for this indexing task."""
        return self.effective_task_id()

    def requires(self):
        return MORJson(date=self.date)



# coding: utf-8

from gluish.benchmark import timed
from gluish.common import OAIHarvestChunk, Executable, Directory
from gluish.format import TSV
from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.path import iterfiles
from gluish.utils import date_range, shellout
from siskin.task import DefaultTask
import datetime
import luigi
import marcx
import os
import pymarc
import re
import tempfile
import time

class HistbestTask(DefaultTask):
    TAG = '036'

    def closest(self):
        return monthly(date=self.date)

class HistbestHarvestChunk(OAIHarvestChunk, HistbestTask):
    """ Harvest all files in chunks. """

    begin = luigi.DateParameter(default=datetime.date(2010, 1, 1))
    end = ClosestDateParameter(default=datetime.date.today())
    url = luigi.Parameter(default="http://histbest.ub.uni-leipzig.de/oai2", significant=False)
    prefix = luigi.Parameter(default="oai_dc")
    collection = luigi.Parameter(default=None, significant=False)
    delay = luigi.Parameter(default=0, significant=False)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml'))

class HistbestHarvest(luigi.WrapperTask, HistbestTask):
    """ Harvest Histbest. """
    begin = luigi.DateParameter(default=datetime.date(2010, 1, 1))
    end = luigi.DateParameter(default=datetime.date.today())
    url = luigi.Parameter(default="http://histbest.ub.uni-leipzig.de/oai2", significant=False)
    prefix = luigi.Parameter(default="oai_dc")
    collection = luigi.Parameter(default=None, significant=False)

    def requires(self):
        """ Only require up to the last full month. """
        begin = datetime.date(self.begin.year, self.begin.month, 1)
        end = datetime.date(self.end.year, self.end.month, 1)
        if end < self.begin:
            raise RuntimeError('Invalid range: %s - %s' % (begin, end))
        dates = date_range(begin, end, 7, 'days')
        for i, _ in enumerate(dates[:-1]):
            yield HistbestHarvestChunk(begin=dates[i], end=dates[i + 1], url=self.url,
                                       prefix=self.prefix, collection=self.collection)

    def output(self):
        return self.input()

class HistbestCombine(HistbestTask):
    """ Concatenate the chunks for a date range into a single file.
    Add an artificial 001 field to all records that is taken directly
    from 856.u, and is either a URN (7407) or a URL (2187). """
    begin = luigi.DateParameter(default=datetime.date(2010, 1, 1))
    date = ClosestDateParameter(default=datetime.date.today())
    url = luigi.Parameter(default="http://histbest.ub.uni-leipzig.de/oai2", significant=False)
    prefix = luigi.Parameter(default="oai_dc")
    collection = luigi.Parameter(default=None, significant=False)

    def requires(self):
        return {'files': HistbestHarvest(begin=self.begin, end=self.date, prefix=self.prefix,
                                         url=self.url, collection=self.collection),
                'apps': [Executable(name='xsltproc'),
                         Executable(name='yaz-marcdump'),
                         Executable(name='marcuniq')]}

    @timed
    def run(self):
        xsl = self.assets("OAIDCtoMARCXML.xsl")
        _, combined = tempfile.mkstemp(prefix='siskin-')
        for target in self.input().get('files'):
            output = shellout("xsltproc {stylesheet} {input} > {output}", stylesheet=xsl, input=target.path)
            output = shellout("yaz-marcdump -f utf-8 -t utf-8 -i marcxml -o marc {input} > {output}", input=output)
            shellout("cat {input} >> {output}", input=output, output=combined)

        with open(combined, 'r') as handle:
            with self.output().open('w') as output:
                reader = pymarc.MARCReader(handle, to_unicode=True, force_utf8=True)
                writer = pymarc.MARCWriter(output)
                for record in reader:
                    record = marcx.FatRecord.from_record(record)
                    if not record.has('001'):
                        if record.has('856.u'):
                            value = list(record.itervalues('856.u'))[0]
                            id = re.sub('(^URN:|^URL:)', '', value).decode('utf-8')
                            record.add('001', data=id)
                            writer.write(record)
                    else:
                        raise RuntimeError('Did not expect IDs in this set.')

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='mrc'))

class HistbestGNDPages(HistbestTask):
    """
    Collect the XML pages, that link to various other entities.
    This is in part realized with a BEACON service:
    http://de.wikipedia.org/wiki/Wikipedia:BEACON
    """

    date = ClosestDateParameter(default=datetime.date.today())
    delay = luigi.IntParameter(default=5, significant=False)
    first = luigi.IntParameter(default=1, significant=False)
    last = luigi.IntParameter(default=5008, significant=False)

    def requires(self):
        return Directory(path=os.path.join(self.taskdir(), str(self.closest())))

    def run(self):
        for i in range(self.first, self.last + 1):
            padded = str(i).zfill(4)
            url = "http://histbest.ub.uni-leipzig.de/receive/UBLHistBestGND_gnd_0000%s?XSL.Style=native" % padded
            output = "%s.xml" % os.path.join(self.input().path, padded)
            if not os.path.exists(output):
                output = shellout("""wget -O "{output}" "{url}" """, url=url, output=output,
                                  ignoremap={8: '404 exit with 8'})
                time.sleep(self.delay)

        with self.output().open('w') as output:
            for path in iterfiles(self.input().path):
                output.write_tsv(path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='filelist'), format=TSV)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301

"""
Formelly known as directory.mozilla.org.

DMOZ uses a hierarchical ontology scheme for organizing site listings.
Listings on a similar topic are grouped into categories
which can then include smaller categories.
"""

from gluish.benchmark import timed
from gluish.common import Executable
from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.task import DefaultTask
import datetime
import luigi

class DMOZTask(DefaultTask):
    """ DMOZ supertask. """
    TAG = 'dmoz'

    def closest(self):
        return monthly(date=self.date)

class DMOZStructureDownload(DMOZTask):
    """ Download structure RDF. """
    date = ClosestDateParameter(default=datetime.date.today())
    url = luigi.Parameter(default='http://rdf.dmoz.org/rdf/structure.rdf.u8.gz', significant=False)

    def requires(self):
        return [Executable(name='curl'), Executable(name='gunzip')]

    @timed
    def run(self):
        output = shellout("""curl "{url}" | gunzip -c > {output}""", url=self.url)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='rdf'))

class DMOZContentDownload(DMOZTask):
    """ Download content RDF. """
    date = ClosestDateParameter(default=datetime.date.today())
    url = luigi.Parameter(default='http://rdf.dmoz.org/rdf/content.rdf.u8.gz', significant=False)

    def requires(self):
        return [Executable(name='curl'), Executable(name='gunzip')]

    @timed
    def run(self):
        output = shellout("""curl "{url}" | gunzip -c > {output}""", url=self.url)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='rdf'))



#!/usr/bin/env python
# coding: utf-8

"""
Configuration keys:

[rvk]

url = http://example.org/path/to/rvk.zip
"""

from gluish.format import TSV
from gluish.parameter import ClosestDateParameter
from gluish.path import iterfiles
from gluish.utils import shellout
from siskin.task import DefaultTask
from siskin.configuration import Config
import collections
import datetime
import luigi
import operator
import os
import tempfile

config = Config.instance()

class RVKTask(DefaultTask):
    """ Default RVK task. """
    TAG = 'rvk'

    def closest(self):
        return datetime.date(2014, 11, 1)

class RVKDownload(RVKTask):
    """ Download and unzip XML dump of RVK. """
    url = luigi.Parameter(default=config.get('rvk', 'url', 'http://example.org/path/to/rvk.zip'))
    date = ClosestDateParameter(default=datetime.date.today())

    def run(self):
        target = os.path.join(self.taskdir(), str(self.date))
        if not os.path.exists(target):
            os.makedirs(target)
        _, stopover = tempfile.mkstemp(prefix='siskin-')
        shellout("wget --retry-connrefused -O {stopover} '{url}' && unzip -o -d {dir} {stopover}", dir=target, stopover=stopover, url=self.url)
        files = list(iterfiles(target))
        if not len(files) == 1:
            raise RuntimeError('more than one file')
        luigi.File(files[0]).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml', digest=True))

class RVKPaths(RVKTask):
    """ Output path to root, one path per line. """

    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return RVKDownload(date=self.date)

    def run(self):
        output = shellout("xsltproc {xsl} {path} > {output}", xsl=self.assets('rvk.xsl'), path=self.input().path)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path())

class RVKNames(RVKTask):
    """ Output path to root, one path per line. """

    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return RVKDownload(date=self.date)

    def run(self):
        output = shellout("xsltproc {xsl} {path} > {output}", xsl=self.assets('rvktsv.xsl'), path=self.input().path)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)

class RVKBSZList(RVKTask):
    """ Extract PPN/RVK from current BSZ index. """

    date = luigi.Parameter(default=datetime.date.today())

    def run(self):
        output = shellout("""estab -indices bsz -f "_id content.936.a" > {output} """)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)

class RVKBSZDistribution(RVKTask):
    """ How many classes are specified. """
    date = luigi.Parameter(default=datetime.date.today())

    def requires(self):
        return RVKBSZList(date=self.date)

    def run(self):
        freq = collections.defaultdict(int)
        with self.input().open() as handle:
            for row in handle.iter_tsv(cols=('ppn', 'rvks')):
                for rvk in row.rvks.split('|'):
                    freq[rvk] += 1

        with self.output().open('w') as output:
            for rvk, count in sorted(freq.iteritems(), key=operator.itemgetter(1), reverse=True):
                output.write_tsv(rvk, count)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103

"""
Gutenberg metadata.
"""

from gluish.esindex import CopyToIndex
from gluish.intervals import weekly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.task import DefaultTask
import datetime
import luigi

class GutenbergTask(DefaultTask):
    TAG = '001'

    def closest(self):
        return weekly(date=self.date)

class GutenbergDownload(GutenbergTask):
    """ Download and extract on the fly. """
    date = ClosestDateParameter(default=datetime.date.today())
    url = luigi.Parameter(default="http://www.gutenberg.org/feeds/catalog.marc.bz2", significant=False)

    def run(self):
        output = shellout("curl -L {url} | bunzip2 -c > {output}", url=self.url)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='mrc'))

class GutenbergJson(GutenbergTask):
    """ Convert Marc to Json. """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return GutenbergDownload(date=self.date)

    def run(self):
        output = shellout("marctojson -m date={date} {input} > {output}",
                          date=self.closest(), input=self.input().path)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj'))

class GutenbergIndex(GutenbergTask, CopyToIndex):
    date = ClosestDateParameter(default=datetime.date.today())

    index = 'gutenberg'
    doc_type = 'title'
    purge_existing_index = True

    mapping = {'title': {'date_detection': False,
                          '_id': {'path': 'content.001'},
                          '_all': {'enabled': True,
                                   'term_vector': 'with_positions_offsets',
                                   'store': True}}}

    def update_id(self):
        """ This id will be a unique identifier for this indexing task."""
        return self.effective_task_id()
    def requires(self):
        return GutenbergJson(date=self.date)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101

"""
PQDT Open.
"""

from gluish.common import OAIHarvestChunk
from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from siskin.task import DefaultTask
import datetime
import luigi

class PQDTTask(DefaultTask):
    TAG = '034'

    def closest(self):
        return monthly(self.date)

class PQDTSync(PQDTTask):
    begin = luigi.DateParameter(default=datetime.date(2000, 1, 1))
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return OAIHarvestChunk(begin=self.begin, end=self.date,
                               url='http://pqdtoai.proquest.com/OAIHandler',
                               prefix='oai_dc')

    def run(self):
        self.input().copy(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path())




# coding: utf-8

"""
arxiv.org

http://arxiv.org/help/oa/index

----

From: https://groups.google.com/forum/#!topic/arxiv-api/aOacIt6KD2E

> the problem is that the API is not the appropriate vehicle to obtain a full
  copy of arXiv metadata. It is intended for interactive queries and search overlays
  and should not be used for full replication. The size of result sets is capped at a large but finite number.
  You should use OAI-PMH instead for your purposes.

"""

from gluish.benchmark import timed
from gluish.common import OAIHarvestChunk
from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout, date_range
from siskin.task import DefaultTask
import datetime
import luigi
import tempfile

class ArxivTask(DefaultTask):
    """ Base task. """

    def closest(self):
        return monthly(date=self.date)

class ArxivHarvestChunk(OAIHarvestChunk, ArxivTask):
    """ Harvest a chunk. """

    begin = luigi.DateParameter(default=datetime.date.today())
    end = ClosestDateParameter(default=datetime.date.today())
    url = luigi.Parameter(default="http://export.arxiv.org/oai2", significant=False)
    prefix = luigi.Parameter(default="oai_dc", significant=False)
    delay = luigi.IntParameter(default=60, significant=False)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml'))

class ArxivHarvest(luigi.WrapperTask, ArxivTask):
    """ Harvest arxiv. """
    begin = luigi.DateParameter(default=datetime.date(2007, 1, 1))
    end = luigi.DateParameter(default=datetime.date.today())
    url = luigi.Parameter(default="http://export.arxiv.org/oai2", significant=False)
    prefix = luigi.Parameter(default="oai_dc", significant=False)

    def requires(self):
        """ Only require up to the last full month. """
        begin = datetime.date(self.begin.year, self.begin.month, 1)
        end = datetime.date(self.end.year, self.end.month, 1)
        if end < self.begin:
            raise RuntimeError('Invalid range: %s - %s' % (begin, end))
        dates = date_range(begin, end, 7, 'days')
        for i, _ in enumerate(dates[:-1]):
            yield ArxivHarvestChunk(begin=dates[i], end=dates[i + 1], url=self.url, prefix=self.prefix)

    def output(self):
        return self.input()

class ArxivCombine(ArxivTask):
    """ Combine the chunks for a date range into a single file.
    The filename will carry a name, that will only include year
    and month for begin and end. """
    begin = luigi.DateParameter(default=datetime.date(2011, 1, 1))
    date = ClosestDateParameter(default=datetime.date.today())
    url = luigi.Parameter(default="http://export.arxiv.org/oai2", significant=False)
    prefix = luigi.Parameter(default="oai_dc", significant=False)

    def requires(self):
        """ monthly updates """
        return ArxivHarvest(begin=self.begin, end=self.date, prefix=self.prefix, url=self.url)

    @timed
    def run(self):
        xsl = self.assets("OAIDCtoMARCXML.xsl")
        _, combined = tempfile.mkstemp(prefix='siskin-')
        for target in self.input().get('files'):
            output = shellout("xsltproc {stylesheet} {input} > {output}",
                              stylesheet=xsl, input=target.path)
            output = shellout("yaz-marcdump -i marcxml -o marc {input} > {output}",
                              input=output)
            shellout("cat {input} >> {output}", input=output, output=combined)

        output = shellout("marcuniq -o {output} {input}", input=combined)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='mrc'))



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301,R0904

"""
Ebrary.

Configuration keys:

[ebrary]

ftp-host = host.name
ftp-username = username
ftp-password = password
ftp-path = /path/to/file.mrc
"""

from gluish.benchmark import timed
from gluish.common import FTPFile
from gluish.esindex import CopyToIndex
from gluish.parameter import ClosestDateParameter
from gluish.path import copyregions
from siskin.task import DefaultTask
from gluish.utils import shellout
from siskin.configuration import Config
import datetime
import luigi
import tempfile

config = Config.instance()

class EbraryTask(DefaultTask):
    TAG = '024'

    def closest(self):
        return datetime.date(2010, 1, 1)

class EbraryCombine(EbraryTask):
    """ FTP mirror and combine relevant files. """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return FTPFile(host=config.get('ebrary', 'ftp-host'),
            username=config.get('ebrary', 'ftp-username'),
            password=config.get('ebrary', 'ftp-password'),
            filepath=config.get('ebrary', 'ftp-path'))

    @timed
    def run(self):
        combined = shellout(r"unzip -p {input} \*.mrc > {output}",
                            input=self.input().path)

        # there is a broken record inside!
        mmap = shellout("""marcmap {input} | LANG=C grep -v ^ebr10661760 \
                           | awk '{{print $2":"$3}}' > {output} """,
                           input=combined)
        # prepare seekmap
        seekmap = []
        with open(mmap) as handle:
            for line in handle:
                offset, length = map(int, line.strip().split(':'))
                seekmap.append((offset, length))

        # create the filtered file
        _, tmp = tempfile.mkstemp(prefix='gluish-')
        with open(tmp, 'w') as output:
            with open(combined) as handle:
                copyregions(handle, output, seekmap)

        output = shellout("""yaz-marcdump -f marc8s -t utf8 -o marc
                          -l 9=97 {input} > {output}""", input=tmp)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='mrc'))

class EbraryJson(EbraryTask):
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return EbraryCombine(date=self.date)

    @timed
    def run(self):
        tmp = shellout("marctojson -m date={date} {input} > {output}",
                       input=self.input().path, date=self.date)
        luigi.File(tmp).move(self.output().fn)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj'))

class EbraryIndex(EbraryTask, CopyToIndex):
    date = ClosestDateParameter(default=datetime.date.today())

    index = 'ebrary'
    doc_type = 'title'
    purge_existing_index = True

    mapping = {'title': {'date_detection': False,
                          '_id': {'path': 'content.001'},
                          '_all': {'enabled': True,
                                   'term_vector': 'with_positions_offsets',
                                   'store': True}}}

    def update_id(self):
        """ This id will be a unique identifier for this indexing task."""
        return self.effective_task_id()

    def requires(self):
        return EbraryJson(date=self.date)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301

"""
IMSLP.

Configuration keys:

[imslp]

scp-src = [user@server:[port]]/path/to/file.txt
"""

from gluish.benchmark import timed
from gluish.esindex import CopyToIndex
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.configuration import Config
from siskin.task import DefaultTask
import datetime
import luigi

config = Config.instance()

class IMSLPTask(DefaultTask):
    TAG = '015'

    def closest(self):
        return datetime.date(2014, 1, 1)

class IMSLPJson(IMSLPTask):
    date = ClosestDateParameter(default=datetime.date.today())

    @timed
    def run(self):
        output = shellout("scp {src} {output}", src=config.get('imslp', 'scp-src'))
        output = shellout("php {script} {input} > {output}", input=output, script=self.assets('imslp2marcxml.php'))
        output = shellout("yaz-marcdump -i marcxml -o marc {input} > {output}", input=output)
        output = shellout("marctojson -m date={date} {input} > {output}", date=self.closest(), input=output)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj'))

class IMSLPIndex(IMSLPTask, CopyToIndex):
    date = ClosestDateParameter(default=datetime.date.today())

    index = 'imslp'
    doc_type = 'title'
    purge_existing_index = True

    mapping = {'title': {'date_detection': False,
                          '_id': {'path': 'content.001'},
                          '_all': {'enabled': True,
                                   'term_vector': 'with_positions_offsets',
                                   'store': True}}}

    def update_id(self):
        """ This id will be a unique identifier for this indexing task."""
        return self.effective_task_id()

    def requires(self):
        return IMSLPJson(date=self.date)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301

from gluish.common import OAIHarvestChunk
from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from siskin.task import DefaultTask
from gluish.utils import shellout
import datetime
import luigi


class TUFGoobiTask(DefaultTask):
    TAG = 'tufgoobi'

    def closest(self):
        return monthly(date=self.date)

class TUFGoobiCombine(TUFGoobiTask):
    """ Digital library via Goobi (#3207) """
    begin = luigi.DateParameter(default=datetime.date(2000, 1, 1))
    date = ClosestDateParameter(default=datetime.date.today())
    prefix = luigi.Parameter(default="oai_dc", significant=False)
    url = luigi.Parameter(default="http://digital.ub.tu-freiberg.de/oai", significant=False)
    collection = luigi.Parameter(default=None, significant=False)

    def requires(self):
        return OAIHarvestChunk(begin=self.begin, end=self.date,
                               prefix=self.prefix, url=self.url,
                               collection=self.collection)

    def run(self):
        output = shellout("xsltproc {stylesheet} {input} > {output}",
                          input=self.input().path,
                          stylesheet=self.assets('OAIDCtoMARCXML.xsl'))
        output = shellout("yaz-marcdump -i marcxml -o marc {input} > {output}",
                          input=output)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='mrc'))



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301

"""
Europeana NT.
"""

from gluish.benchmark import timed
from gluish.format import TSV
from gluish.path import iterfiles
from gluish.utils import shellout
from siskin.task import DefaultTask
import glob
import luigi
import os
import shutil
import tempfile

class EuropeanaTask(DefaultTask):
    TAG = 'europeana'

class EuropeanaDownload(EuropeanaTask):
    """ Download Europeana dump. """
    base = luigi.Parameter(default='http://data.europeana.eu/download', significant=False)
    version = luigi.Parameter(default='2.0')
    format = luigi.Parameter(default='nt')

    @timed
    def run(self):
        target = os.path.join(self.taskdir(), self.version, self.format)
        if not os.path.exists(target):
            os.makedirs(target)
        url = os.path.join(self.base, self.version, "datasets", self.format)
        stopover = tempfile.mkdtemp(prefix='siskin-')
        shellout("""wget -q -nd -P {directory} -rc -np -A.{format}.gz '{url}'""",
                    url=url, directory=stopover, format=self.format)
        for path in glob.glob(unicode(os.path.join(stopover, '*'))):
            dst = os.path.join(target, os.path.basename(path))
            if not os.path.exists(dst):
                # this is atomic given path and target are on the same device
                shutil.move(path, target)
        with self.output().open('w') as output:
            for path in iterfiles(target, fun=lambda p: p.endswith('nt.gz')):
                output.write_tsv(self.version, self.format, path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='filelist'), format=TSV)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301

"""
Bibliographie des Musikschrifttums online (BMS online).

http://www.musikbibliographie.de

Configuration keys:

[bms]

ftp-host = host.name
ftp-username = username
ftp-password = password
ftp-filepath = /path/to/file

"""

from gluish.benchmark import timed
from gluish.common import FTPFile
from gluish.esindex import CopyToIndex
from gluish.intervals import yearly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.configuration import Config
from siskin.task import DefaultTask
import datetime
import luigi

config = Config.instance()

class BMSTask(DefaultTask):
    """ BMS base task. """
    TAG = '011'

    def closest(self):
        return yearly(date=self.date)

class BMSImport(BMSTask):
    """ Fetch bmsall.xml from the FTP server. """

    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        host = config.get('bms', 'ftp-host')
        username = config.get('bms', 'ftp-username')
        password = config.get('bms', 'ftp-password')
        filepath = config.get('bms', 'ftp-filepath')
        return FTPFile(host=host, username=username, password=password,
                       filepath=filepath)

    @timed
    def run(self):
        self.input().move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml'))

class BMSMarc(BMSTask):
    """ Convert the XML to MARC. """

    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return BMSImport(date=self.date)

    @timed
    def run(self):
        output = shellout("yaz-marcdump -i marcxml -o marc {input} > {output}",
                          input=self.input().path, ignoremap={5: 'INVESTIGATE'})
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='mrc'))

class BMSJson(BMSTask):
    """ Convert the combined MARC file to a single JSON file."""
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return BMSMarc(date=self.date)

    @timed
    def run(self):
        tmp = shellout("marctojson -m date={date} {input} > {output}",
                       input=self.input().path, date=self.date)
        luigi.File(tmp).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj'))

class BMSIndex(BMSTask, CopyToIndex):

    date = ClosestDateParameter(default=datetime.date.today())

    index = 'bms'
    doc_type = 'title'
    purge_existing_index = True

    mapping = {'title': {'date_detection': False,
                          '_id': {'path': 'content.001'},
                          '_all': {'enabled': True,
                                   'term_vector': 'with_positions_offsets',
                                   'store': True}}}

    def update_id(self):
        """ This id will be a unique identifier for this indexing task. """
        return self.effective_task_id()

    def requires(self):
        return BMSJson(date=self.date)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301,C0103,W0614,W0401

from gluish.format import TSV
from gluish.intervals import yearly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.task import DefaultTask
import BeautifulSoup
import datetime
import HTMLParser
import luigi
import operator


class PrefixCCTask(DefaultTask):
    TAG = 'prefixcc'

    def closest(self):
        return yearly(date=self.date)

class PrefixCCDownload(PrefixCCTask):
    """ Download common ns prefixes, about 1k. """
    date = ClosestDateParameter(default=datetime.date.today())
    url = luigi.Parameter(default="http://prefix.cc/popular/all.rdf", significant=False)

    def run(self):
        output = shellout("wget --retry-connrefused '{url}' -O {output}", url=self.url)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='html'))

class PrefixCCParse(PrefixCCTask):
    """ Parse out. """

    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return PrefixCCDownload()

    def run(self):
        with self.input().open() as handle:
            soup = BeautifulSoup.BeautifulSoup(handle.read())
        lines = map(lambda s: HTMLParser.HTMLParser().unescape(s.strip()),
                    soup.find("pre").text.split("\n"))

        shortest = {}        
        for line in sorted(lines):
            if not line.startswith('xmlns:'):
                continue
            parts = line.split("=")
            if not len(parts) == 2:
                continue
            head, rest = parts
            head = head.replace('xmlns:', '')
            rest = rest.strip('"')
            if rest in shortest:
                if len(head) < len(shortest[rest]):
                    shortest[rest] = head
            else:
                shortest[rest] = head
        
        with self.output().open('w') as output:
            for rest, head in sorted(shortest.iteritems(), key=operator.itemgetter(1)):
                output.write_tsv(head, rest)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,R0904,E1103

"""
IMDB.
"""

from gluish.common import FTPMirror
from gluish.intervals import hourly
from siskin.task import DefaultTask
import luigi

class IMDBTask(DefaultTask):
    TAG = 'imdb'

class IMDBSync(IMDBTask):

    indicator = luigi.Parameter(default=hourly(fmt='%s'))

    def requires(self):
        return FTPMirror(host='ftp.fu-berlin.de',
                         username='anonymous',
                         password='',
                         base='/pub/misc/movies/database/')

    def run(self):
        self.input().move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path())



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301

"""
World factbook related tasks.
"""

from gluish.intervals import yearly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.task import DefaultTask
import datetime
import luigi

class FactbookTask(DefaultTask):
    """ Base task for factbook. """
    TAG = 'factbook'

    def closest(self):
        return yearly(self.date)

class FactbookXML(FactbookTask):
    """ Download factbook XML version. There may be better versions. """
    date = ClosestDateParameter(default=datetime.date.today())

    def run(self):
        url = 'http://jmatchparser.sourceforge.net/factbook/data/factbook.xml.gz'
        output = shellout("curl --connect-timeout 10 {url} | gunzip -c > {output}", url=url)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml'))



#!/usr/bin/env python
# coding: utf-8

"""
GBI publisher.

[gbi]

ftp-host = host.name
ftp-username = username
ftp-password = password
ftp-path = /
ftp-pattern = some*glob*pattern.zip
"""

from gluish.benchmark import timed
from gluish.common import FTPMirror, Executable
from gluish.format import TSV
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.configuration import Config
from siskin.task import DefaultTask
import datetime
import luigi
import tempfile

config = Config.instance()

class GBITask(DefaultTask):
    TAG = '048'

    def closest(self):
        return datetime.date(2015, 6, 1)

class GBISync(GBITask):
    """ Sync. """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return FTPMirror(host=config.get('gbi', 'ftp-host'),
                         username=config.get('gbi', 'ftp-username'),
                         password=config.get('gbi', 'ftp-password'),
                         pattern=config.get('gbi', 'ftp-pattern'))

    @timed
    def run(self):
        self.input().move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)

# --*-- commercial break --*--
#
# This is a sketch, how the task below might be implemented in a type-safe
# language with a fictional library called lynd.
# Go: 598 chars, Python: 723 chars.
#

# package gbi

# type Group struct {
#     Date  lynd.Date   `default:"2010-01-01"`
#     Group lynd.String `default:"wiwi" help:"wiwi, fzs, sowi, recht"`
# }

# func (task Groups) Requires() interface{} {
#     return GBIInventory{Date: task.Date}
# }

# func (task Groups) Run() error {
#     for fields := range lynd.In(task).StringFields() {
#         group := strings.Split(fields[0], "/")[-2]
#         if group == task.Group {
#             lynd.Out(task).WriteTabs(fields[0])
#         }
#     }
#     return lynd.Out(task).Flush()
# }

# func (task Groups) Output() Target {
#     return lynd.AutoTargetFormat(task, lynd.TSV)
# }

class GBIGroup(GBITask):
    """ Find groups (subdirs). These are not part of the metadata. """
    date = ClosestDateParameter(default=datetime.date.today())
    group = luigi.Parameter(default='wiwi', description='wiwi, fzs, sowi, recht')

    def requires(self):
        return GBISync(date=self.date)

    def run(self):
        with self.input().open() as handle:
            with self.output().open('w') as output:
                for row in handle.iter_tsv(cols=('path',)):
                    group = row.path.split('/')[-2]
                    if group == self.group:
                        output.write_tsv(row.path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml'), format=TSV)

class GBIXMLGroup(GBITask):
    """
    Extract all XML files and concat them. Inject a <x-group> tag, which
    carries the "collection" from the directory name into the metadata.
    """
    date = ClosestDateParameter(default=datetime.date.today())
    group = luigi.Parameter(default='wiwi', description='wiwi, fzs, sowi, recht')

    def requires(self):
       return GBIGroup(date=self.date, group=self.group)

    @timed
    def run(self):
        _, stopover = tempfile.mkstemp(prefix='siskin-')
        with self.input().open() as handle:
            for row in handle.iter_tsv(cols=('path',)):
                shellout("""unzip -p {path} \*.xml 2> /dev/null |
                            iconv -f iso-8859-1 -t utf-8 |
                            LC_ALL=C grep -v "^<\!DOCTYPE GENIOS PUBLIC" |
                            LC_ALL=C sed -e 's@<?xml version="1.0" encoding="ISO-8859-1" ?>@@g' |
                            LC_ALL=C sed -e 's@</Document>@<x-group>{group}</x-group></Document>@' >> {output}""",
                         output=stopover, path=row.path, group=self.group,
                         ignoremap={1: 'OK', 9: 'ignoring broken zip'})
        luigi.File(stopover).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml'), format=TSV)

class GBIXML(GBITask):
    """
    Extract all XML files and concat them. Inject a <x-group> tag, which
    carries the "collection" from the directory name into the metadata.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        """ Note: ingore broken 'sowi' for now. """
        return [GBIXMLGroup(date=self.date, group=group) for group in ['wiwi', 'fzs', 'recht']]

    @timed
    def run(self):
        _, stopover = tempfile.mkstemp(prefix='siskin-')
        for target in self.input():
            shellout("cat {input} >> {output}", input=target.path, output=stopover)
        luigi.File(stopover).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml'), format=TSV)

class GBIIntermediateSchema(GBITask):
    """
    Convert GBI to intermediate format via span.
    If group is 'all', create intermediate schema of all groups.
    """

    date = ClosestDateParameter(default=datetime.date.today())
    group = luigi.Parameter(default='all', description='wiwi, fzs, sowi, recht')

    def requires(self):
        span = Executable(name='span-import', message='http://git.io/vI8NV')
        if self.group == "all":
            return {'span': span, 'file': GBIXML(date=self.date)}
        else:
            return {'span': span, 'file': GBIXMLGroup(date=self.date, group=self.group)}

    @timed
    def run(self):
        output = shellout("span-import -i genios {input} > {output}", input=self.input().get('file').path)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj'))

class GBIISSNList(GBITask):
    """ A list of JSTOR ISSNs. """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return GBIIntermediateSchema(date=self.date)

    @timed
    def run(self):
        _, stopover = tempfile.mkstemp(prefix='siskin-')
        shellout("""jq -r '.["rft.issn"][]' {input} 2> /dev/null >> {output} """, input=self.input().path, output=stopover)
        shellout("""jq -r '.["rft.eissn"][]' {input} 2> /dev/null >> {output} """, input=self.input().path, output=stopover)
        output = shellout("""sort -u {input} > {output} """, input=stopover)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301

"""
B3Kat open data. http://www.b3kat.de/
"""

from gluish.benchmark import timed
from gluish.common import Executable
from gluish.format import TSV
from gluish.intervals import quarterly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout, random_string
from siskin.task import DefaultTask
import datetime
import luigi
import random
import requests
import tempfile
import time
import urllib
import urlparse

class B3KatTask(DefaultTask):
    """ Base task for http://www.b3kat.de/ related things. """
    TAG = 'b3kat'

    def closest(self):
        return quarterly()

class B3KatDownload(B3KatTask):
    """
    Download dump, extract and concat on the fly.
    Note: Output will be about 80G in size.

    Download URL is hardcoded and points to:

    http://lod.b3kat.de/download/lod.b3kat.de.part{part}.ttl.gz
    """
    date = ClosestDateParameter(default=datetime.date.today())
    template = luigi.Parameter(default='http://lod.b3kat.de/download/lod.b3kat.de.part{part}.ttl.gz',
                               significant=False)

    def requires(self):
        return Executable(name='curl')

    @timed
    def run(self):
        _, stopover = tempfile.mkstemp(prefix='siskin-')
        for i in range(27):
            url = self.template.format(part=i)
            shellout("""curl --retry 1 --compress "{url}" >> {output}""",
                     url=url, output=stopover)
        luigi.File(stopover).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)

class B3KatSameAs(B3KatTask):
    """ Extract the sameAs information in an ugly way. """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return {'content': B3KatDownload(date=self.date),
                'apps': Executable(name='serdi', message='http://drobilla.net/software/serd/')}

    def run(self):
        """ TODO: try https://github.com/miku/nttoldj """
        output = shellout("""serdi -b -f -i turtle -o ntriples {input} | LANG=C grep "owl#sameAs" > {output}""",
                          input=self.input().get('content').path)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='n3'))

class B3KatExtractRelation(B3KatTask):
    """
    Extract MARC relations.
    List of relators: http://www.loc.gov/marc/relators/relaterm.html
    """
    date = ClosestDateParameter(default=datetime.date.today())
    relator = luigi.Parameter(default="http://id.loc.gov/vocabulary/relators/aut",
                              description='http://www.loc.gov/marc/relators/relaterm.html')

    def requires(self):
        return {'content': B3KatDownload(date=self.date),
                'apps': Executable(name='serdi', message='http://drobilla.net/software/serd/')}

    def run(self):

        output = shellout("""serdi -b -f -i turtle -o ntriples {input} | LANG=C grep "{relator}" > {output}""",
                          relator=self.relator, input=self.input().get('content').path)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='n3', digest=True))

class B3KatShadowHarvest(B3KatTask):
    """
    Shadow harvest bug. Investigate bug, where subsequent GET requests
    to the same URL return different results.
    """

    tag = luigi.Parameter(default=random_string(), description='run on every call')

    # request parameters
    scheme = luigi.Parameter(default='http', significant=False)
    netloc = luigi.Parameter(default='bvbr.bib-bvb.de:8991', significant=False)
    urlpath = luigi.Parameter(default='F', significant=False)
    func = luigi.Parameter(default='service', significant=False)
    doc_library = luigi.Parameter(default='BVB01', significant=False)
    doc_number = luigi.Parameter(default='020028928', significant=False)
    service_type = luigi.Parameter(default='MEDIA', significant=False)

    loop = luigi.IntParameter(default=100, description='repeat request this many times',
                              significant=False)

    def run(self):
        query = urllib.urlencode(dict(func=self.func, doc_library=self.doc_library,
                                      doc_number=self.doc_number, service_type=self.service_type))
        url = urlparse.urlunparse((self.scheme, self.netloc, self.urlpath, '', query, ''))
        with self.output().open('w') as output:
            for _ in range(self.loop):
                try:
                    r = requests.get(url, timeout=20)
                    output.write(r.text)
                    output.write("\n\n\n\n")
                    time.sleep(random.random())
                except Exception as err:
                    self.logger.error(err)
                    break

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301

"""
BVB via OAI.
"""

from gluish.benchmark import timed
from gluish.common import OAIHarvestChunk
from gluish.esindex import CopyToIndex
from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.utils import date_range, shellout
from siskin.task import DefaultTask
import datetime
import luigi
import tempfile

class BVBTask(DefaultTask):
    TAG = '012'

    def closest(self):
        return monthly(date=self.date)

class BVBHarvestChunk(OAIHarvestChunk, BVBTask):
    """ Harvest all files in chunks. """

    begin = luigi.DateParameter(default=datetime.date.today())
    end = ClosestDateParameter(default=datetime.date.today())
    url = luigi.Parameter(default="http://bvbr.bib-bvb.de:8991/aleph-cgi/oai/oai_opendata.pl", significant=False)
    prefix = luigi.Parameter(default="marc21", significant=False)
    collection = luigi.Parameter(default="OpenData", significant=False)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml'))

class BVBHarvest(luigi.WrapperTask, BVBTask):
    """ Harvest BVB, cut all dates to the first of month. Example:
    2013-05-12 .. 2013-09-11 would be translated to
    2013-05-01 .. 2013-09-01

    Note: About 10G in total?
    """
    begin = luigi.DateParameter(default=datetime.date(2011, 1, 1))
    end = luigi.DateParameter(default=datetime.date.today())
    url = luigi.Parameter(default="http://bvbr.bib-bvb.de:8991/aleph-cgi/oai/oai_opendata.pl", significant=False)
    prefix = luigi.Parameter(default="marc21", significant=False)
    collection = luigi.Parameter(default="OpenData", significant=False)

    def requires(self):
        """ Only require up to the last full month. """
        begin = datetime.date(self.begin.year, self.begin.month, 1)
        end = datetime.date(self.end.year, self.end.month, 1)
        if end < self.begin:
            raise RuntimeError('Invalid range: %s - %s' % (begin, end))
        dates = date_range(begin, end, 7, 'days')
        for i, _ in enumerate(dates[:-1]):
            yield BVBHarvestChunk(begin=dates[i], end=dates[i + 1], url=self.url,
                                  prefix=self.prefix, collection=self.collection)

    def output(self):
        return self.input()

class BVBCombine(BVBTask):
    """ Combine the chunks for a date range into a single file.
    The filename will carry a name, that will only include year
    and month for begin and end. """
    begin = luigi.DateParameter(default=datetime.date(2011, 1, 1))
    date = ClosestDateParameter(default=datetime.date.today())
    url = luigi.Parameter(default="http://bvbr.bib-bvb.de:8991/aleph-cgi/oai/oai_opendata.pl", significant=False)
    prefix = luigi.Parameter(default="marc21", significant=False)
    collection = luigi.Parameter(default="OpenData", significant=False)

    def requires(self):
        """ monthly updates """
        return BVBHarvest(begin=self.begin, end=self.date, prefix=self.prefix,
                          url=self.url, collection=self.collection)

    @timed
    def run(self):
        _, combined = tempfile.mkstemp(prefix='tasktree-')
        for target in self.input():
            tmp = shellout("""yaz-marcdump -f utf-8 -t utf-8 -i
                              marcxml -o marc {input} > {output}""",
                              input=target.fn, ignoremap={5: 'TODO: fix this'})
            shellout("cat {input} >> {output}", input=tmp, output=combined)
        output = shellout("marcuniq -o {output} {input}", input=combined)
        luigi.File(output).move(self.output().fn)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='mrc'))

class BVBJson(BVBTask):
    """ Convert to JSON. """

    begin = luigi.DateParameter(default=datetime.date(2011, 1, 1))
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return BVBCombine(begin=self.begin, date=self.date)

    def run(self):
        tmp = shellout("marctojson -m date={date} {input} > {output}",
                       input=self.input().fn, date=self.date)
        luigi.File(tmp).move(self.output().fn)

    def output(self):
        """ Use monthly files. """
        return luigi.LocalTarget(path=self.path(ext='ldj'))

class BVBIndex(BVBTask, CopyToIndex):
    begin = luigi.DateParameter(default=datetime.date(2011, 1, 1))
    date = ClosestDateParameter(default=datetime.date.today())

    index = "bvb"
    doc_type = "title"
    purge_existing_index = True

    mapping = {'title': {'date_detection': False,
                          '_id': {'path': 'content.001'},
                          '_all': {'enabled': True,
                                   'term_vector': 'with_positions_offsets',
                                   'store': True}}}

    def update_id(self):
        """ This id will be a unique identifier for this indexing task."""
        return self.effective_task_id()

    def requires(self):
        return BVBJson(begin=self.begin, date=self.date)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103

from gluish.benchmark import timed
from gluish.common import OAIHarvestChunk
from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout, date_range
from siskin.task import DefaultTask
import datetime
import luigi
import tempfile

class PerseeTask(DefaultTask):
    TAG = '039'

    def closest(self):
        return monthly(date=self.date)

class PerseeHarvestChunk(OAIHarvestChunk, PerseeTask):
    """ Harvest all files in chunks. """

    begin = luigi.DateParameter(default=datetime.date.today())
    end = ClosestDateParameter(default=datetime.date.today())
    url = luigi.Parameter(default="http://oai.persee.fr/c/ext/prescript/oai", significant=False)
    prefix = luigi.Parameter(default="marc", significant=False)
    collection = luigi.Parameter(default=None)
    delay = luigi.IntParameter(default=10, significant=False)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml'))

class PerseeHarvest(luigi.WrapperTask, PerseeTask):
    """ Harvest Histbest. """
    begin = luigi.DateParameter(default=datetime.date(2010, 1, 1))
    end = luigi.DateParameter(default=datetime.date.today())
    url = luigi.Parameter(default="http://oai.persee.fr/c/ext/prescript/oai", significant=False)
    prefix = luigi.Parameter(default="marc", significant=False)
    collection = luigi.Parameter(default=None)
    delay = luigi.IntParameter(default=10, significant=False)

    def requires(self):
        """ Only require up to the last full month. """
        begin = datetime.date(self.begin.year, self.begin.month, 1)
        end = datetime.date(self.end.year, self.end.month, 1)
        if end < self.begin:
            raise RuntimeError('Invalid range: %s - %s' % (begin, end))
        dates = date_range(begin, end, 7, 'days')
        for i, _ in enumerate(dates[:-1]):
            yield PerseeHarvestChunk(begin=dates[i], end=dates[i + 1], url=self.url,
                                    prefix=self.prefix, collection=self.collection, delay=self.delay)

    def output(self):
        return self.input()

class PerseeCombine(PerseeTask):
    """ Combine the chunks for a date range into a single file.
    The filename will carry a name, that will only include year
    and month for begin and end. """
    begin = luigi.DateParameter(default=datetime.date(2010, 1, 1))
    date = ClosestDateParameter(default=datetime.date.today())
    url = luigi.Parameter(default="http://oai.persee.fr/c/ext/prescript/oai", significant=False)
    prefix = luigi.Parameter(default="marc", significant=False)
    collection = luigi.Parameter(default=None)
    delay = luigi.IntParameter(default=10, significant=False)

    def requires(self):
        """ monthly updates """
        return PerseeHarvest(begin=self.begin, end=self.date, prefix=self.prefix,
                            url=self.url, collection=self.collection, delay=self.delay)

    @timed
    def run(self):
        _, combined = tempfile.mkstemp(prefix='siskin-')
        for target in self.input():
            tmp = shellout("""yaz-marcdump -f utf-8 -t utf-8 -i
                              marcxml -o marc {input} > {output}""",
                              input=target.fn, ignoremap={5: 'TODO: fix this'})
            shellout("cat {input} >> {output}", input=tmp, output=combined)
        luigi.File(combined).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='mrc'))



# coding: utf-8

from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.task import DefaultTask
import datetime
import luigi

class NowpubTask(DefaultTask):
    TAG = "nowpub"

    def closest(self):
        return monthly(date=self.date)

class NowpubMarc(NowpubTask):
    """ Download zipfile an combine marc records. """

    date = ClosestDateParameter(default=datetime.date.today())

    def run(self):
        output = shellout("wget -O {output} http://www.nowpublishers.com/Public-Content/marc-records/fntall.zip")
        output = shellout("unzip -p {input} *mrc > {output}", input=output)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='mrc'))

class NowpubJson(NowpubTask):
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return NowpubMarc(date=self.date)

    def run(self):
        output = shellout("marctojson {input} > {output}", input=self.input().path)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj'))



#!/usr/bin/env python

"""
The EZB.

Configuration keys:

[ezb]

# url to uncompressed TSV file
url = http://example.org/file/61f1456.tab
"""

from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.configuration import Config
from siskin.task import DefaultTask
from siskin.utils import ElasticsearchMixin
import csv
import datetime
import json
import luigi

config = Config.instance()

class EZBTask(DefaultTask):
    """ EZB related tasks. """
    TAG = 'ezb'

    def closest(self):
        return datetime.date(2015, 1, 1)

class EZBTSVImport(EZBTask):
    """ Download TSV from URL. """
    date = ClosestDateParameter(default=datetime.date.today())

    def run(self):
        output = shellout("""wget --retry-connrefused -O {output} "{url}" """, url=config.get('ezb', 'url'))
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path())

class EZBJson(EZBTask):
    """ Convert TSV to LDJ. """
    date = ClosestDateParameter(default=datetime.date.today())
    skiphead = luigi.IntParameter(default=5, significant=False)
    encoding = luigi.Parameter(default='iso-8859-1', significant=False)

    def requires(self):
        return EZBTSVImport(date=self.date)

    def run(self):
        cols = ['ezb-id', 'title', 'color', 'publisher', 'subject', 'eissn', 'pissn',
                'zdb-id', 'frontdoor-url', 'type', 'price-type', 'access', 'link',
                'anchor', 'first-year', 'first-volume', 'first-issue', 'last-year',
                'last-volume', 'last-issue', 'moving-wall', 'available']
        with self.input().open() as handle:
            with self.output().open('w') as output:
                for i in range(self.skiphead):
                    handle.next()
                reader = csv.reader(handle, delimiter='\t')
                for row in reader:
                    r = [c.decode(self.encoding) for c in row]
                    output.write(json.dumps(dict(zip(cols, r))))
                    output.write("\n")

    def output(self):
        return luigi.LocalTarget(path=self.path())

class EZBIndex(EZBTask, ElasticsearchMixin):
    """ Index EZB data. """
    date = ClosestDateParameter(default=datetime.date.today())
    skiphead = luigi.IntParameter(default=5, significant=False)
    encoding = luigi.Parameter(default='iso-8859-1', significant=False)
    index = luigi.Parameter(default='ezb')

    def requires(self):
        return EZBJson(date=self.date, skiphead=self.skiphead, encoding=self.encoding)

    def run(self):
        shellout("curl -XDELETE {host}:{port}/{index}", host=self.es_host, port=self.es_port, index=self.index)
        shellout("esbulk -verbose -index {index} {input}", index=self.index, input=self.input().path)
        with self.output().open('w'):
            pass

    def output(self):
        return luigi.LocalTarget(path=self.path())



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301

from gluish.benchmark import timed
from gluish.common import OAIHarvestChunk
from gluish.esindex import CopyToIndex
from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.task import DefaultTask
import datetime
import luigi
import pymarc

class DOABTask(DefaultTask):
    TAG = '026'

    def closest(self):
        return monthly(date=self.date)

class DOABCombine(DOABTask):
    """ Combine the chunks for a date range into a single file.
    The filename will carry a name, that will only include year
    and month for begin and end. """

    begin = luigi.DateParameter(default=datetime.date(2010, 1, 1))
    date = ClosestDateParameter(default=datetime.date.today())
    prefix = luigi.Parameter(default="marcxml", significant=False)
    url = luigi.Parameter(default="http://www.doabooks.org/oai", significant=False)
    collection = luigi.Parameter(default=None, significant=False)

    def requires(self):
        return OAIHarvestChunk(begin=self.begin, end=self.date,
                               prefix=self.prefix, url=self.url,
                               collection=self.collection)

    @timed
    def run(self):
        tmp = shellout("""yaz-marcdump -l 9=97 -f UTF8 -t UTF8
                          -i marcxml -o marc {input} > {output}""",
                          input=self.input().path, ignoremap={5: 'INVESTIGATE'})

        # filter dups, bad ids and unreadable records ...
        # TODO: make this possible with gomarckit as well
        seen = set()
        with open(tmp) as handle:
            with self.output().open('w') as output:
                reader = pymarc.MARCReader(handle, to_unicode=True)
                writer = pymarc.MARCWriter(output)
                while True:
                    try:
                        record = reader.next()
                        record_id = record['001'].value()
                        if record_id in ('15270', '15298', '15318', '15335'):
                            self.logger.debug("Skipping {0}".format(record_id))
                            continue
                        if not record_id in seen:
                            writer.write(record)
                            seen.add(record_id)
                        else:
                            self.logger.debug("Skipping duplicate: {0}".format(record_id))
                    except pymarc.exceptions.RecordDirectoryInvalid as err:
                        self.logger.warn(err)
                    except StopIteration:
                        break

    def output(self):
        """ Use monthly files. """
        return luigi.LocalTarget(path=self.path(digest=True, ext='mrc'))

class DOABJson(DOABTask):
    """ Convert to JSON. """

    begin = luigi.DateParameter(default=datetime.date(2010, 1, 1))
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return DOABCombine(begin=self.begin, date=self.date)

    def run(self):
        output = shellout("marctojson -m date={date} {input} > {output}",
                          input=self.input().path, date=self.date)
        luigi.File(output).move(self.output().path)

    def output(self):
        """ Use monthly files. """
        return luigi.LocalTarget(path=self.path(ext='ldj'))

class DOABIndex(DOABTask, CopyToIndex):
    begin = luigi.DateParameter(default=datetime.date(2010, 1, 1))
    date = ClosestDateParameter(default=datetime.date.today())

    index = "doab"
    doc_type = "title"
    purge_existing_index = True

    mapping = {'title': {'date_detection': False,
                          '_id': {'path': 'content.001'},
                          '_all': {'enabled': True,
                                   'term_vector': 'with_positions_offsets',
                                   'store': True}}}

    def update_id(self):
        """ This id will be a unique identifier for this indexing task."""
        return self.effective_task_id()

    def requires(self):
        return DOABJson(begin=self.begin, date=self.date)



# coding: utf-8
# pylint: disable=C0103,W0232
"""
Define a siskin wide task with artefacts under core.home directory.
"""

from gluish.task import BaseTask
from gluish.utils import shellout
from siskin.configuration import Config
import logging
import os
import random
import siskin

config = Config.instance()

class DefaultTask(BaseTask):
    """ A base task that sets its base directory based on config value. """
    BASE = config.get('core', 'home', Config.NO_DEFAULT)

    def assets(self, path):
        """ Return the absolute path to the asset. `path` is the relative path
        below the assets root dir. """
        return os.path.join(os.path.dirname(__file__), 'assets', path)

    @property
    def logger(self):
        # logging uses singleton internally, so no worries
        return logging.getLogger('siskin')

    @property
    def logfile(self):
        """ A logfile per task for stderr redirects. """
        name = self.taskdir().split('/')[-1]
        return os.path.join(config.get('core', 'logdir', siskin.DEFAULT_LOG_DIR), '%s.log' % name)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,R0904

from gluish.benchmark import timed
from gluish.common import OAIHarvestChunk
from gluish.esindex import CopyToIndex
from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout, date_range
from siskin.task import DefaultTask
import BeautifulSoup
import collections
import datetime
import hashlib
import json
import luigi
import marcx
import pymarc
import tempfile

class QucosaTask(DefaultTask):
    TAG = '022'

    def closest(self):
        return monthly(date=self.date)

class QucosaHarvestChunk(OAIHarvestChunk, QucosaTask):
    """ Harvest all files in chunks. """

    begin = luigi.DateParameter(default=datetime.date.today())
    end = luigi.DateParameter(default=datetime.date.today())
    url = luigi.Parameter(default="http://www.qucosa.de/oai", significant=False)
    prefix = luigi.Parameter(default="xMetaDissPlus")
    collection = luigi.Parameter(default=None)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml', digest=True))

class QucosaJsonChunk(QucosaTask):
    """ Convert Qucosa XML into JSON by hand. Any better way?
    Maybe sth. like https://github.com/gawel/pyquery """
    begin = luigi.DateParameter(default=datetime.date(2010, 1, 1))
    end = luigi.DateParameter(default=datetime.date.today())

    def requires(self):
        return QucosaHarvestChunk(begin=self.begin, end=self.end)

    @timed
    def run(self):
        docs = []

        with self.input().open() as handle:
            soup = BeautifulSoup.BeautifulStoneSoup(handle.read())
            for record in soup.findAll('record'):
                doc = collections.defaultdict(list)

                # header
                header = record.find('header')
                if header.get('status') == 'deleted':
                    continue

                # IDs
                doc['oai'] = header.find('identifier').text
                doc['id'] = doc['oai'].replace('oai:qucosa.de:', '')

                # the whole metadata element
                metadata = record.find('metadata')
                diss = metadata.find('xmetadiss:xmetadiss')
                if not diss:
                    raise ValueError('qucosa without xmetadiss')

                # title
                if diss.find('dc:title'):
                    doc['title'] = diss.find('dc:title').text
                else:
                    self.logger.error("Qucosa w/o title: {0}".format(doc.get('id', 'NA')))
                    # raise ValueError('qucosa without title')

                # subtitle
                alternative = diss.find('dcterms:alternative')
                if alternative and (alternative.get("xsi:type") ==
                                    'ddb:talternativeISO639-2'):
                    doc['alt'] = alternative.text

                # source
                source = diss.find('dc:source')
                if source and source.get('xsi:type') == 'ddb:noScheme':
                    doc['source'] = source.text

                # creators
                for creator in diss.findAll('dc:creator'):
                    forename = creator.find('pc:forename', recursive=True)
                    surname = creator.find('pc:surname', recursive=True)
                    doc['creator'].append("%s %s" % (forename.text, surname.text))

                # subjects
                for subject in diss.findAll('dc:subject'):
                    doc[subject['xsi:type']].append(subject.text)

                # abstracts
                for abstract in diss.findAll('dc:abstract'):
                    doc['abstract'].append(abstract.text)

                # tocs
                for toc in diss.findAll('dcterms:tableOfContents'):
                    doc['toc'].append(toc.text)

                # publisher
                for publisher in diss.findAll('dc:publisher'):
                    name = publisher.find('cc:name', recursive=True)
                    place = publisher.find('cc:place', recursive=True)
                    doc['publisher'].append("%s %s" % (name.text, place.text))

                # contributor
                for contributor in diss.findAll('dc:contributor'):
                    forename = contributor.find('pc:forename', recursive=True)
                    surname = contributor.find('pc:surname', recursive=True)
                    role = contributor.get('thesis:role')
                    doc['contributor'].append({
                        'name': '%s %s' % (forename.text, surname.text),
                        'role': role
                    })

                # publication type
                if diss.find('dc:type').get('xsi:type') == 'dini:PublType':
                    doc['pubtype'] = diss.find('dc:type').text

                # date issued
                try:
                    doc['issued'] = diss.find('dcterms:issued').text
                except AttributeError:
                    pass

                # language
                try:
                    doc['lang'] = diss.find('dc:language').text
                except AttributeError:
                    pass

                # urns
                for dcid in diss.findAll('dc:identifier'):
                    if dcid.get('xsi:type') == 'urn:nbn':
                        doc['urn'] = dcid.text
                        doc['id'] = dcid.text # override id

                # transfer (broken urls?)
                for transfer in diss.findAll('ddb:transfer'):
                    if transfer.get('ddb:type') == 'dcterms:URI':
                        doc['transfer'] = transfer.text

                # medium
                for medium in diss.findAll('dcterms:medium'):
                    if medium.get('xsi:type') == 'dcterms:IMT':
                        doc['mimetype'] = medium.text

                doc['filenumber'] = int(diss.find('ddb:filenumber').text)

                doc['filesize'] = 0 # adding up
                for fileprop in diss.findAll('ddb:fileproperties'):
                    try:
                        doc['filesize'] += int(fileprop.get('ddb:filesize', 0))
                        doc['filename'].append(fileprop.get('ddb:filename'))
                    except ValueError:
                        self.logger.warn("Empty/wrong filesize in {0}".format(doc['id']))

                for ispartof in diss.findAll('dcterms:ispartof'):
                    if ispartof.get('xsi:type') == 'ddb:noScheme':
                        doc['ispartof_title'].append(ispartof.text)
                    if ispartof.get('xsi:type') == 'ddb:ISSN':
                        doc['ispartof_issn'].append(ispartof.text)
                    if ispartof.get('xsi:type') == 'ddb:URI':
                        doc['ispartof_uri'].append(ispartof.text.replace(
                                                   'http://nbn-resolving.de/', ''))

                docs.append(doc)

        # dump as json, one document per line
        with self.output().open('w') as output:
            for doc in docs:
                output.write(json.dumps(doc))
                output.write('\n')

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='json'))

class QucosaMarcChunk(QucosaTask):
    """ Convert a jsonified qucosa XML into MARC.
    TODO: use external standalone python script as asset. """

    begin = luigi.DateParameter(default=datetime.date(2010, 1, 1))
    end = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return QucosaJsonChunk(begin=self.begin, end=self.end)

    @timed
    def run(self):
        records = []
        with self.input().open() as handle:
            for line in handle:
                record = marcx.FatRecord(to_unicode=True, force_utf8=True)
                doc = json.loads(line)

                record.add('001', doc['id'])

                if 'lang' in doc:
                    record.add('041', a=doc['lang'])

                if 'issued' in doc:
                    deof = datetime.datetime.strptime(doc['issued'], '%Y-%m-%d').strftime('%y%m%d')
                    year = datetime.datetime.strptime(doc['issued'], '%Y-%m-%d').strftime('%Y')
                    record.add('008',
                               data='{deof}s{year}    xx{skip}{lang}'.format(
                               deof=deof, year=year, skip=18 * ' ',
                               lang=doc.get('lang', '   ')))

                for i, name in enumerate(doc.get('creator', [])):
                    if i == 0:
                        record.add('100', a=name, e='author')
                        continue
                    record.add('700', a=name, e='author')

                for contributor in doc.get('contributor', []):
                    record.add('700', a=contributor.get('name'),
                               e=contributor.get('role'))

                if 'title' in doc:
                    if 'alt' in doc:
                        record.add('245', a=doc['title'], b=doc['alt'])
                    else:
                        record.add('245', a=doc['title'])

                for publisher in doc.get('publisher', []):
                    record.add('260', b=publisher)

                if 'issued' in doc:
                    year = datetime.datetime.strptime(doc['issued'], '%Y-%m-%d').strftime('%Y')
                    record.add('260', c=year, _9=doc['issued'])

                if 'source' in doc:
                    record.add('490', a=doc['source'])

                for toc in doc.get('toc', []):
                    hashed = hashlib.sha1(toc.encode('utf-8')).hexdigest()
                    record.add('520', indicators='2 ', _9=hashed)

                for abstract in doc.get('abstract', []):
                    hashed = hashlib.sha1(abstract.encode('utf-8')).hexdigest()
                    record.add('520', indicators='3 ', _9=hashed)

                if 'pubtype' in doc:
                    record.add('500', a=doc['pubtype'])

                if 'url' in doc:
                    record.add('856', indicators='40', u=doc['url'])

                if 'transfer' in doc:
                    subs = collections.defaultdict(list)
                    if 'mimetype' in doc:
                        subs['q'] = doc.get('mimetype')
                    if 'filesize' in doc and doc.get('filesize') > 0:
                        subs['s'] = str(doc.get('filesize'))
                    for fn in doc.get('filename', [])[:9]:
                        if doc.get('fn'):
                            subs['f'].append(doc.get(fn))

                    if doc.get('filenumber') > 1:
                        filenames = sorted(doc.get('filename'), reverse=True,
                                           key=lambda fn: fn.endswith('.pdf'))
                        fnstr = ', '.join(filenames[:3])
                        if len(filenames) > 0:
                            fnstr += ', ...'
                        subs['_3'] = '%s (%s files: %s)' % (doc['transfer'],
                                     doc['filenumber'], fnstr)

                    subs['u'] = doc['transfer']
                    record.add('856', indicators='41', **subs)

                for subject in doc.get('xMetaDiss:noScheme', []):
                    if subject is not None:
                        record.add('650', indicators=' 4', a=subject)

                for subject in doc.get('dcterms:DDC', []):
                    record.add('082', indicators='0 ', a=subject)

                for subject in doc.get('xMetaDiss:RVK', []):
                    record.add('084', indicators='  ', _2='rvk', a=subject)

                if 'ispartof_uri' in doc:
                    subs = {}
                    if 'ispartof_title' in doc:
                        parts = doc.get('ispartof_title')[0].split(';')
                        if len(parts) > 1:
                            t = ';'.join(parts[:-1]).strip()
                            v = parts[-1]
                            if t and v:
                                subs['t'] = t
                                subs['v'] = v
                        else:
                            subs['a'] = doc.get('ispartof_title')[0]
                    if 'ispartof_issn':
                        subs['x'] = doc.get('ispartof_issn')[0]
                    subs['_6'] = doc.get('ispartof_uri')[0]
                    record.add('830', indicators=' 0', **subs)

                records.append(record)

        with self.output().open('w') as output:
            writer = pymarc.MARCWriter(output)
            for record in records:
                writer.write(record)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='mrc'))

class QucosaCombine(QucosaTask):
    """ Harvest qucosa, cut all dates to the first of month. Example:
    2013-05-12 .. 2013-09-11 would be translated to
    2013-05-01 .. 2013-09-01"""

    begin = luigi.DateParameter(default=datetime.date(2010, 1, 1))
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        """ Only require up to the last full month. """
        date = self.closest()
        if date < self.begin:
            raise RuntimeError('invalid range: %s - %s' % (self.begin, self.date))
        dates = date_range(self.begin, date, 1, 'months')
        for i, _ in enumerate(dates[:-1]):
            yield QucosaMarcChunk(begin=dates[i], end=dates[i + 1])

    @timed
    def run(self):
        _, combined = tempfile.mkstemp(prefix='tasktree-')
        for target in self.input():
            shellout("cat {input} >> {output}", input=target.path,
                     output=combined)
        output = shellout("marcuniq -o {output} {input}", input=combined)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='mrc'))

class QucosaJson(QucosaTask):
    """ Convert to JSON. """

    begin = luigi.DateParameter(default=datetime.date(2010, 1, 1))
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return QucosaCombine(begin=self.begin, date=self.date)

    def run(self):
        output = shellout("marctojson -m date={date} {input} > {output}",
                          input=self.input().path, date=self.date)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='json'))

class QucosaIndex(QucosaTask, CopyToIndex):
    begin = luigi.DateParameter(default=datetime.date(2010, 1, 1))
    date = ClosestDateParameter(default=datetime.date.today())

    index = 'qucosa'
    doc_type = 'title'
    purge_existing_index = True

    mapping = {'title': {'date_detection': False,
                          '_id': {'path': 'content.001'},
                          '_all': {'enabled': True,
                                   'term_vector': 'with_positions_offsets',
                                   'store': True}}}

    def update_id(self):
        """ This id will be a unique identifier for this indexing task."""
        return self.effective_task_id()

    def requires(self):
        return QucosaJson(begin=self.begin, date=self.date)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301

"""
A place for one time tasks and quick write-ups.
"""

from elasticsearch import helpers as eshelpers
from gluish.format import TSV
from gluish.utils import itervalues
from siskin.task import DefaultTask
import datetime
import elasticsearch
import json
import luigi
import marcx
import re

class AdhocTask(DefaultTask):
    """ Tasks that do not fit elsewhere. """
    TAG = 'adhoc'

class DOIList(AdhocTask):
    """
    For all open indices, run a heuristic DOI query and dump a TSV of
    (index, id, DOI, year, title and subtitle) tuples.
    """
    date = luigi.DateParameter(default=datetime.date.today())

    encoding = luigi.Parameter(default='utf-8', significant=False)
    na = luigi.Parameter(default='NOT_AVAILABLE', significant=False)
    scroll = luigi.Parameter(default='10m', significant=False)
    timeout = luigi.IntParameter(default=30, significant=False)

    def run(self):
        es = elasticsearch.Elasticsearch(timeout=self.timeout)
        hits = eshelpers.scan(es, {'query': {"regexp": {"_all": "10\\.[0-9]{4,}"}}}, scroll=self.scroll)
        with self.output().open('w') as output:
            for hit in hits:
                doc_id, index = hit.get('_id'), hit.get('_index')
                content = hit.get('_source').get('content')
                matches = re.findall(r"10\.[0-9]{4,}/[^ \t\"]{3,}", json.dumps(content))
                for candidate in set(matches):
                    doc = marcx.marcdoc(hit)
                    title = (doc.values('245.a') or [self.na])[0]
                    subtitle = (doc.values('245.b') or [self.na])[0]
                    pubyear = (doc.values('260.c') or [self.na])[0]
                    output.write_tsv(index,
                                     doc_id,
                                     candidate.encode(self.encoding),
                                     pubyear.encode(self.encoding),
                                     title.encode(self.encoding),
                                     subtitle.encode(self.encoding))

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301

"""
Munzinger.

Configuration keys:

[munz]

src = [user@server:[port]]/path/to/directory
"""

from gluish.common import Directory
from gluish.format import TSV
from gluish.parameter import ClosestDateParameter
from gluish.path import iterfiles
from gluish.utils import shellout
from siskin.configuration import Config
from siskin.task import DefaultTask
import datetime
import luigi

config = Config.instance()

class MunzTask(DefaultTask):
    TAG = '027'

    def closest(self):
        return datetime.date(2014, 1, 1)

class MunzSync(MunzTask):
    """ Just copy over the zip files. """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return Directory(path=self.taskdir())

    def run(self):
        shellout("rsync -avz {src} {dst}", src=config.get('munz', 'src'), dst=self.taskdir())
        with self.output().open('w') as output:
            for path in iterfiles(self.taskdir(), fun=lambda p: '-luigi-tmp-' not in p):
                output.write_tsv(path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='filelist'), format=TSV)



#!/usr/bin/env python
# coding: utf-8

"""
Homepage: http://www.textgrid.de/ueber-textgrid/digitale-bibliothek/
License: http://creativecommons.org/licenses/by/3.0/de/
"""

from gluish.common import Directory
from gluish.format import TSV
from gluish.parameter import ClosestDateParameter
from gluish.path import iterfiles
from gluish.utils import shellout
from siskin.task import DefaultTask
import datetime
import luigi
import os

class TextGridTask(DefaultTask):
    TAG = 'textgrid'

    def closest(self):
        return datetime.date(2014, 11, 1)

class TextGridDownload(TextGridTask):
    """ Download of various corpus dumps. """
    date = ClosestDateParameter(default=datetime.date.today())
    corpus = luigi.Parameter(default='all', description='all, v1, v2')

    def requires(self):
        return {'dir': Directory(path=os.path.join(self.taskdir(), self.corpus, str(self.closest())))}

    def run(self):
        filemap = {'all': 'http://www.textgrid.de/fileadmin/digitale-bibliothek/literatur.zip',
                   'v1': 'http://www.textgrid.de/fileadmin/digitale-bibliothek/literatur-nur-texte-1.zip',
                   'v2': 'http://www.textgrid.de/fileadmin/digitale-bibliothek/literatur-nur-texte-2.zip'}

        if self.corpus not in filemap:
            raise RuntimeError('available corpus ids: all, v1, v2')

        output = shellout("wget --retry-connrefused '{url}' -O {output}", url=filemap[self.corpus])
        shellout("unzip -d '{dir}' {input}", dir=self.input().get('dir').path, input=output)
        with self.output().open('w') as output:
            for path in iterfiles(self.input().get('dir').path):
                output.write_tsv(path.encode('utf-8'))

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,R0904,E1103,C0301

"""
[jstor]

ftp-host = host.name
ftp-username = username
ftp-password = password
ftp-path = /
ftp-pattern = *
"""

from gluish.benchmark import timed
from gluish.common import FTPMirror, Executable
from gluish.format import TSV
from gluish.intervals import weekly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout, nwise
from siskin.configuration import Config
from siskin.task import DefaultTask
import datetime
import itertools
import luigi
import pipes
import tempfile

config = Config.instance()

class JstorTask(DefaultTask):
    """ Jstor base. """
    TAG = 'jstor'

    def closest(self):
        return weekly(self.date)

class JstorPaths(JstorTask):
    """ Sync. """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return FTPMirror(host=config.get('jstor', 'ftp-host'),
                         username=config.get('jstor', 'ftp-username'),
                         password=config.get('jstor', 'ftp-password'),
                         pattern=config.get('jstor', 'ftp-pattern'))

    def run(self):
        self.input().move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)

class JstorMembers(JstorTask):
    """ Extract a full list of archive members. """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return JstorPaths(date=self.date)

    @timed
    def run(self):
        _, stopover = tempfile.mkstemp(prefix='siskin-')
        with self.input().open() as handle:
            for row in handle.iter_tsv(cols=('path',)):
                shellout(""" unzip -l {input} | grep "xml$" | awk '{{print "{input}\t"$4}}' >> {output} """,
                         preserve_whitespace=True, input=row.path, output=stopover)
        luigi.File(stopover).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)

class JstorLatestMembers(JstorTask):
    """ Find the latest archive members for all article ids. """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return JstorMembers(date=self.date)

    @timed
    def run(self):
        """ Expect input to be sorted, so tac will actually be a perfect rewind. """
        output = shellout("tac {input} | sort -S 50% -u -k2,2 | sort -S 50% -k1,1 > {output}", input=self.input().path)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)

class JstorXML(JstorTask):
    """
    Create an latest snapshot of the latest data.
    """
    date = ClosestDateParameter(default=datetime.date.today())
    batch = luigi.IntParameter(default=512, significant=False)

    def requires(self):
        return JstorLatestMembers(date=self.date)

    @timed
    def run(self):
        _, stopover = tempfile.mkstemp(prefix='siskin-')
        with self.input().open() as handle:
            groups = itertools.groupby(handle.iter_tsv(cols=('archive', 'member')), lambda row: row.archive)
            for archive, items in groups:
                for chunk in nwise(items, n=self.batch):
                    margs = " ".join(["'%s'" % item.member.replace('[', r'\[').replace(']', r'\]') for item in chunk])
                    shellout("unzip -qq -c {archive} {members} >> {output}", archive=archive, members=margs, output=stopover)

        luigi.File(stopover).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml'), format=TSV)

class JstorIntermediateSchema(JstorTask):
    """ Convert to intermediate format via span. """

    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
       return {'span': Executable(name='span-import', message='http://git.io/vI8NV'),
               'file': JstorXML(date=self.date)}

    @timed
    def run(self):
        output = shellout("span-import -i jstor {input} > {output}", input=self.input().get('file').path)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj'))

class JstorISSNList(JstorTask):
    """ A list of JSTOR ISSNs. """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return JstorIntermediateSchema(date=self.date)

    @timed
    def run(self):
        _, stopover = tempfile.mkstemp(prefix='siskin-')
        shellout("""jq -r '.["rft.issn"][]' {input} 2> /dev/null >> {output} """, input=self.input().path, output=stopover)
        shellout("""jq -r '.["rft.eissn"][]' {input} 2> /dev/null >> {output} """, input=self.input().path, output=stopover)
        output = shellout("""sort -u {input} > {output} """, input=stopover)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)



# coding: utf-8

"""
DeGruyter task.

[degruyter]

ftp-host = host.name
ftp-username = username
ftp-password = password
ftp-path = /
ftp-pattern = some*glob*pattern.zip

"""

from gluish.benchmark import timed
from gluish.common import FTPMirror, Executable
from gluish.format import TSV
from gluish.intervals import daily
from gluish.parameter import ClosestDateParameter
from gluish.path import iterfiles
from gluish.utils import shellout
from siskin.configuration import Config
from siskin.task import DefaultTask
import datetime
import luigi
import re
import shutil
import tempfile

config = Config.instance()

class DegruyterTask(DefaultTask):
    TAG = 'degruyter'

    def closest(self):
        return datetime.date(2015, 4, 1)

class DegruyterPaths(DegruyterTask):
    """ A list of Degruyter ile paths (via FTP). """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        host = config.get('degruyter', 'ftp-host')
        username = config.get('degruyter', 'ftp-username')
        password = config.get('degruyter', 'ftp-password')
        base = config.get('degruyter', 'ftp-path')
        pattern = config.get('degruyter', 'ftp-pattern')
        return FTPMirror(host=host, username=username, password=password,
                         base=base, pattern=pattern)

    @timed
    def run(self):
        self.input().move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext="filelist"), format=TSV)

class DegruyterMembers(DegruyterTask):
    """ Extract a full list of archive members. """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return DegruyterPaths(date=self.date)

    @timed
    def run(self):
        _, stopover = tempfile.mkstemp(prefix='siskin-')
        with self.input().open() as handle:
            for row in handle.iter_tsv(cols=('path',)):
                shellout(""" unzip -l {input} | grep "xml$" | awk '{{print "{input}\t"$4}}' >> {output} """,
                         preserve_whitespace=True, input=row.path, output=stopover)
        luigi.File(stopover).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)

class DegruyterXML(DegruyterTask):
    """
    Extract all XML files from Jstor dump. TODO(miku): Check all subdirs, not just SSH.
    """
    date = ClosestDateParameter(default=datetime.date.today())
    group = luigi.Parameter(default='SSH', description='Nationallizenz_Zeitschriften, Nationallizenz_Jahrbuecher')

    def requires(self):
        return DegruyterPaths(date=self.date)

    @timed
    def run(self):
        _, stopover = tempfile.mkstemp(prefix='siskin-')
        with self.input().open() as handle:
            for row in handle.iter_tsv(cols=('path',)):
                if not '/%s/' % self.group in row.path:
                    continue
                shellout("unzip -p {path} \*.xml 2> /dev/null >> {output}", output=stopover, path=row.path,
                         ignoremap={1: 'OK', 9: 'skip corrupt file'})
        luigi.File(stopover).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml'), format=TSV)

class DegruyterIntermediateSchema(DegruyterTask):
    """ Convert to intermediate format via span. """

    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
       return {'span': Executable(name='span-import', message='http://git.io/vI8NV'),
               'file': DegruyterXML(date=self.date)}

    @timed
    def run(self):
        output = shellout("span-import -i degruyter {input} > {output}", input=self.input().get('file').path)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj'))

class DegruyterISSNList(DegruyterTask):
    """ List of ISSNs. """

    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
       return DegruyterIntermediateSchema(date=self.date)

    @timed
    def run(self):
        _, stopover = tempfile.mkstemp(prefix='siskin-')
        shellout("""jq -r '.["rft.issn"][]' {input} 2> /dev/null >> {output} """, input=self.input().path, output=stopover)
        shellout("""jq -r '.["rft.eissn"][]' {input} 2> /dev/null >> {output} """, input=self.input().path, output=stopover)
        output = shellout("""sort -u {input} > {output} """, input=stopover)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(), format=TSV)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101

"""
PA Online

Configuration keys:

[pao]

scp-src = [user@server:[port]]/path/to/file/*mrc
deletions = [user@server:[port]]/path/to/paodel.mrc

"""

from gluish.benchmark import timed
from gluish.common import Executable
from gluish.esindex import CopyToIndex
from gluish.parameter import ClosestDateParameter
from gluish.path import iterfiles
from gluish.utils import shellout
from siskin.configuration import Config
from siskin.task import DefaultTask
import datetime
import luigi
import re
import tempfile

config = Config.instance()

class PAOTask(DefaultTask):
    TAG = '007'

    def closest(self):
        """ One time data source. """
        return datetime.date(2014, 1, 1)

class PAOSync(PAOTask):
    date = ClosestDateParameter(default=datetime.date.today())

    @timed
    def run(self):
        stopover = tempfile.mkdtemp(prefix='siskin-')
        shellout("scp {origin} {stopover}".format(origin=config.get('pao', 'scp-src'), stopover=stopover))
        _, combined = tempfile.mkstemp(prefix='siskin-')
        for path in iterfiles(directory=stopover,
                              fun=lambda path: re.search(r'pao[\d].mrc', path)):
            shellout("cat {path} >> {output}", path=path, output=combined)
        luigi.File(combined).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='mrc'))

class PAODeletions(PAOTask):
    date = ClosestDateParameter(default=datetime.date.today())

    @timed
    def run(self):
        output = shellout("scp {deletions} {output}", deletions=config.get('pao', 'deletions'))
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='mrc'))

class PAOJson(PAOTask):
    """ Convert to JSON, respect deletions. """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return {'additions': PAOSync(date=self.date),
                'deletions': PAODeletions(date=self.date),
                'apps': [Executable(name='marctotsv'),
                         Executable(name='marcuniq')]
        }

    @timed
    def run(self):
        # sanitize
        additions = shellout("""yaz-marcdump -f iso-8859-1 -t utf-8
                             -i marc -o marc {input} > {output}""",
                             input=self.input().get('additions').path)
        deletions = shellout("""yaz-marcdump -f iso-8859-1 -t utf-8
                             -i marc -o marc {input} > {output}""",
                             input=self.input().get('deletions').path)

        # filter deletions
        excludes = shellout("""marctotsv {input} 001 > {output} """,
                            input=deletions)
        output = shellout("""marcuniq -x {excludes} -o {output} {input} """,
                       excludes=excludes, input=additions)

        # to json
        output = shellout("marctojson -m date={date} {input} > {output}",
                          date=self.closest(), input=output)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj'))

class PAOIndex(PAOTask, CopyToIndex):
    date = ClosestDateParameter(default=datetime.date.today())

    index = 'pao'
    doc_type = 'title'
    purge_existing_index = True

    mapping = {'title': {'date_detection': False,
                          '_id': {'path': 'content.001'},
                          '_all': {'enabled': True,
                                   'term_vector': 'with_positions_offsets',
                                   'store': True}}}

    def update_id(self):
        """ This id will be a unique identifier for this indexing task."""
        return self.effective_task_id()

    def requires(self):
        return PAOJson(date=self.date)



# coding: utf-8

from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.task import DefaultTask
import datetime
import luigi

class DBLPTask(DefaultTask):
    TAG = 'dblp'

    def closest(self):
        return monthly(date=self.date)

class DBLPDownload(DBLPTask):
    """ Download file and extract. """
    url = luigi.Parameter(default='http://dblp.uni-trier.de/xml/dblp.xml.gz', significant=False)
    date = ClosestDateParameter(default=datetime.date.today())

    def run(self):
        output = shellout("curl -L {url} | gunzip -c > {output}", url=self.url)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml'))



# coding: utf-8

from gluish.common import OAIHarvestChunk
from gluish.parameter import ClosestDateParameter
from gluish.utils import date_range, shellout
from gluish.intervals import weekly
from siskin.task import DefaultTask
import datetime
import luigi
import tempfile

class ThiemeTask(DefaultTask):
    """ Thieme connect. """
    TAG = 'thieme'

    def closest(self):
        return weekly(date=self.date)

class ThiemeHarvestChunk(OAIHarvestChunk, ThiemeTask):
    """ Harvest all files in chunks. """

    begin = luigi.DateParameter(default=datetime.date.today())
    end = ClosestDateParameter(default=datetime.date.today())
    url = luigi.Parameter(default="https://www.thieme-connect.de/oai/provider", significant=False)
    prefix = luigi.Parameter(default="tm", significant=True)
    collection = luigi.Parameter(default='journalarticles')
    delay = luigi.IntParameter(default=2, significant=False)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml'))

class ThiemeHarvest(luigi.WrapperTask, ThiemeTask):
    """ Harvest Thieme. """
    begin = luigi.DateParameter(default=datetime.date(1970, 1, 1))
    end = ClosestDateParameter(default=datetime.date.today())
    url = luigi.Parameter(default="https://www.thieme-connect.de/oai/provider", significant=False)
    prefix = luigi.Parameter(default="tm", significant=True)
    collection = luigi.Parameter(default='journalarticles')
    delay = luigi.IntParameter(default=2, significant=False)

    def requires(self):
        """ Only require up to the last full month. """
        begin = datetime.date(self.begin.year, self.begin.month, 1)
        end = datetime.date(self.end.year, self.end.month, 1)
        if end < self.begin:
            raise RuntimeError('Invalid range: %s - %s' % (begin, end))
        dates = date_range(begin, end, 1, 'months')
        for i, _ in enumerate(dates[:-1]):
            yield ThiemeHarvestChunk(begin=dates[i], end=dates[i + 1], url=self.url,
                                     prefix=self.prefix, collection=self.collection, delay=self.delay)

    def output(self):
        return self.input()

class ThiemeCombine(ThiemeTask):
    """ Combine files. Simple cat files and leave span-import the rest. """

    begin = luigi.DateParameter(default=datetime.date(1970, 1, 1))
    end = ClosestDateParameter(default=datetime.date.today())
    url = luigi.Parameter(default="https://www.thieme-connect.de/oai/provider", significant=False)
    prefix = luigi.Parameter(default="tm", significant=True)
    collection = luigi.Parameter(default='journalarticles')
    delay = luigi.IntParameter(default=2, significant=False)

    def requires(self):
        return ThiemeHarvest(begin=self.begin, end=self.end, url=self.url, prefix=self.prefix, collection=self.collection, delay=self.delay)

    def run(self):
        _, stopover = tempfile.mkstemp(prefix='siskin-')
        for target in self.input():
            shellout("cat {input} >> {output}", input=target.path, output=stopover)
        luigi.File(stopover).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(digest=True, ext="xml"))



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301

"""
Gallica.
"""

from __future__ import print_function
from gluish.benchmark import timed
from gluish.esindex import CopyToIndex
from gluish.intervals import monthly
from gluish.oai import oai_harvest
from gluish.parameter import ClosestDateParameter
from gluish.path import iterfiles
from gluish.utils import date_range, shellout
from siskin.task import DefaultTask
import BeautifulSoup
import codecs
import collections
import datetime
import itertools
import json
import luigi
import marcx
import re
import tempfile

class BNFTask(DefaultTask):
    TAG = '020'

    def closest(self):
        return monthly(date=self.date)

class BNFHarvestChunk(BNFTask):
    """ Harvest all files in chunks. """
    begin = luigi.DateParameter(default=datetime.date(2011, 1, 1))
    end = ClosestDateParameter(default=datetime.date.today())
    prefix = luigi.Parameter(default="oai_dc")
    collection = luigi.Parameter(default="gallica:typedoc:partitions")

    @timed
    def run(self):
        """ Harvest files for a certain timeframe in a temporary
        directory, then combine all records into a single file. """
        stopover = tempfile.mkdtemp(prefix='tasktree-')
        oai_harvest(url="http://oai.bnf.fr/oai2/OAIHandler",
                    begin=self.begin, end=self.end, prefix=self.prefix,
                    directory=stopover, collection=self.collection)
        with self.output().open('w') as output:
            output.write("""<collection
                xmlns="http://www.openarchives.org/OAI/2.0/"
                xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">""")
            for path in iterfiles(stopover):
                with open(path) as handle:
                    soup = BeautifulSoup.BeautifulStoneSoup(handle.read())
                    for record in soup.findAll('record'):
                        output.write(str(record)) # or unicode?
            output.write('</collection>\n')

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml', digest=True))

class BNFJsonChunk(BNFTask):
    """ Combine the chunks for a date range into a single file.
    The filename will carry a name, that will only include year
    and month for begin and end."""
    begin = luigi.DateParameter(default=datetime.date(2011, 1, 1))
    end = ClosestDateParameter(default=datetime.date.today())
    prefix = luigi.Parameter(default="oai_dc")
    collection = luigi.Parameter(default="gallica:typedoc:partitions")

    def requires(self):
        return BNFHarvestChunk(begin=self.begin, end=self.end,
                               prefix=self.prefix, collection=self.collection)

    @timed
    def run(self):
        """ TODO: this is much much too long. """
        fr_en_map = {
            u'domaine public': u'public domain',
            u'musique manuscrite': u'manuscript music',
            u"Notice d'ensemble : ": '',
            u'Notice du catalogue : ': '',
            u'Notice de recueil : ': '',
        }

        irregulars = {
            u'Cellarius, Henri (neveu18..-18..? ; chorégraphe). Chorégraphe':
                u'Cellarius, Henri (18..-18..? ; chorégraphe). Chorégraphe',
            u'Sophocle (0496?-0406 av. J.-C.). Auteur adapté':
                u'Sophocle (-0496?-0406). Auteur adapté',
            u'Horace (0065-0008 av. J.-C.). Auteur du texte':
                u'Horace (-0065-0008). Auteur du texte',
            u'Lacour (17..-ap. 1826). Librettiste':
                u'Lacour (17..-1826). Librettiste',
            u'Eschyle (0525?-0456 av. J.-C.). Auteur adapté':
                u'Eschyle (-0525?-0456). Auteur adapté',
            u'Benaut, Josse-François-Joseph (1743-1794). Compositeur. 1100':
                u'Benaut, Josse-François-Joseph (1743-1794). Compositeur',
            u'Capron, Nicolas (1740?-1784). Compositeur. 1100':
                u'Capron, Nicolas (1740?-1784). Compositeur'
        }

        def parse_people(s):
            if s in irregulars.keys():
                for match, replacement in irregulars.iteritems():
                    s = s.replace(match, replacement)
            if s.endswith(')'):
                if re.search(r'[0-9]\)$', s):
                    s += '. TYPE'
            if s in (u'Agence de publicité', u'Dédicataire', u'Parolier',
                     u'Compositeur'):
                s = u'Anonyme. ' + s

            # person name: no digits
            a = r'(?P<a>[^0-9]+)'
            # (year-year ; foo)
            # d = r'\((?P<d>[0-9\.\-\? ]+)(?: ; [^0-9]+)?\)'
            # (foo ; year-year ; bar)
            # d = r'\(([^0-9]+ ; )?(?P<d>[0-9\.\-\? ]+)( ; [^0-9]+)?\)'
            # (foo ; year-year ; bar)
            d = r'\(([^0-9]+ ; )?(?P<d>[0-9\.\-\? ]+)( ; .*?)?\)'
            # type of work: no digits
            e = r'(?P<e>[^0-9]+)'
            # type of work: no digits
            # e = r'(?P<e>[^0-9]{5,})'

            # name (years). type
            test = re.match(r'%s( %s)?(\. %s)$' % (a, d, e), s)

            if test:
                # io.append({ 'l':line, 'm':test.groupdict() })
                # k['t'].append(test.groupdict())
                sf = test.groupdict()
            else:
                # io.append({ 'l':line })
                # k['f'].append(line)
                # no match: assume person name alone
                sf = {'a': s, 'd': '', 'e' : ''}
                # no match: assume person name alone
                # sf = {'a': line, 'd': '', 'e':''}
            return sf

        docs = []

        with self.input().open() as handle:
            soup = BeautifulSoup.BeautifulStoneSoup(handle.read())

        for record in soup.findAll('record'):
            doc = collections.defaultdict(list)

            header = record.find('header')

            # identifier
            doc['oai'] = header.find('identifier').text
            doc['id'] = doc['oai'].split('/')[-1] # too short?

            if header.get('status') == 'deleted':
                self.logger.debug("Skipping deleted: {0}".format(doc['oai']))
                continue

            metadata = record.find('metadata')

            # title
            for title in metadata.findAll('dc:title'):
                doc['title'].append(title.text)

            # creator
            for creator in metadata.findAll('dc:creator'):
                doc['creator'].append(parse_people(creator.text))

            # subject
            for subject in metadata.findAll('dc:subject'):
                doc['subject'].append(subject.text)

            # publisher
            for publisher in metadata.findAll('dc:publisher'):
                doc['publisher'].append(publisher.text)

            # pubdate
            for pubdate in metadata.findAll('dc:date'):
                doc['pubdate'].append(pubdate.text)

            # description
            for description in metadata.findAll('dc:description'):
                doc['description'].append(description.text)

            # format
            for format in metadata.findAll('dc:format'):
                if format.text.startswith('application/'):
                    doc['mimetype'].append(format.text)
                else:
                    doc['format'].append(format.text)

            # relation
            relations = set()
            for rel in metadata.findAll('dc:relation'):
                translation = fr_en_map.get(rel.text, rel.text)
                if translation:
                    relations.add(translation)
            doc['relation'] = list(relations)

            # score and pubtype
            scores, pubtype = set(), set()
            for dctype in metadata.findAll('dc:type'):
                translated = fr_en_map.get(dctype.text, '')
                if translated == 'manuscript music':
                    scores.add(translated)
                else:
                    pubtype.add(translated)

            doc['score'] = list(scores)
            doc['pubtype'] = list(pubtype)

            # dc identifier
            for dcid in metadata.findAll('dc:identifier'):
                if dcid.text.startswith('http://'):
                    doc['url'].append(dcid.text)
                else:
                    doc['identifier'].append(dcid.text)

            for source in metadata.findAll('dc:source'):
                doc['source'].append(source.text)

            rights = set()
            for right in metadata.findAll('dc:rights'):
                rights.add(fr_en_map.get(right.text))

            docs.append(doc)

        # dump as json, one document per line
        with self.output().open('w') as output:
            for doc in docs:
                output.write(json.dumps(doc))
                output.write('\n')

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='json', digest=True))

class BNFMarcChunk(BNFTask):
    """ Convert a JSON snippet into MARC. """
    begin = luigi.DateParameter(default=datetime.date(1970, 1, 1))
    end = ClosestDateParameter(default=datetime.date.today())
    prefix = luigi.Parameter(default="oai_dc")
    collection = luigi.Parameter(default="gallica:typedoc:partitions")

    def requires(self):
        return BNFJsonChunk(begin=self.begin, end=self.end,
                            prefix=self.prefix, collection=self.collection)

    @timed
    def run(self):
        _, stopover = tempfile.mkstemp(prefix="tasktree-")
        with self.input().open() as handle:
            with codecs.open(stopover, 'w', 'utf-8') as output:
                for line in handle:
                    record = marcx.FatRecord(to_unicode=True, force_utf8=True)
                    doc = json.loads(line.encode('utf-8'))

                    record.add('001', data=doc['id'])
                    record.add('037', n=doc['oai'])
                    record.add('042', a='dc')

                    people = itertools.chain(doc.get('creator', []),
                                             doc.get('contributors', []))
                    for i, person in enumerate(people):
                        a = person.get('a') or ''
                        d = person.get('d') or ''
                        e = person.get('e') or ''
                        if i == 0:
                            # only add the first creator as 100
                            record.add('100', a=a, d=d, e=e)
                        else:
                            # all others need to go to 700
                            record.add('700', a=a, d=d, e=e)

                    for i, title in enumerate(doc.get('title', [])):
                        if i == 0:
                            record.add('245', a=title)
                        else:
                            record.add('242', a=title)

                    for publisher in doc.get('publisher', []):
                        record.add('260', b=publisher)

                    for pubdate in doc.get('pubdate', []):
                        record.add('260', c=pubdate)

                    for description in doc.get('description', []):
                        record.add('520', a=description)

                    for score in doc.get('score', []):
                        record.add('254', a=score)

                    for pubtype in doc.get('pubtype', []):
                        record.add('655', indicators='7 ', a=pubtype)

                    if 'url' in doc:
                        subs = collections.defaultdict(list)
                        if 'mimetype' in doc:
                            subs['q'] = doc.get('mimetype')[0]
                        if 'rights' in doc:
                            subs['z'] = doc.get('rights')[0]
                        subs['u'] = doc.get('url')[0]
                        record.add('856', indicators='40', **subs)

                    for subject in doc.get('subject', []):
                        record.add('609', a=subject)

                    for ident in doc.get('identifier', []):
                        record.add('609', a=ident)

                    if len(doc.get('relation', [])) > 0:
                        record.add('786', i=[r for r in doc.get('relation', [])])

                    output.write(record.as_marc().decode('utf-8'))

        luigi.File(stopover).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='mrc', digest=True))

class BNFMarcHarvest(luigi.WrapperTask, BNFTask):
    """ Harvest BNF, cut all dates to the first of month. Example:
    2013-05-12 .. 2013-09-11 would be translated to
    2013-05-01 .. 2013-09-01"""
    begin = luigi.DateParameter(default=datetime.date(2011, 1, 1))
    end = ClosestDateParameter(default=datetime.date.today())
    prefix = luigi.Parameter(default="oai_dc")
    collection = luigi.Parameter(default="gallica:typedoc:partitions")

    def requires(self):
        """ Only require up to the last full month. """
        begin = datetime.date(self.begin.year, self.begin.month, 1)
        end = datetime.date(self.end.year, self.end.month, 1)
        if end < self.begin:
            raise RuntimeError('Invalid range: %s - %s' % (begin, end))
        dates = date_range(begin, end, 1, 'months')
        for i, _ in enumerate(dates[:-1]):
            yield BNFMarcChunk(begin=dates[i], end=dates[i + 1],
                               prefix=self.prefix, collection=self.collection)

    def output(self):
        return self.input()

class BNFCombine(BNFTask):
    """ Combine all MARC files into one. """
    begin = luigi.DateParameter(default=datetime.date(2011, 1, 1))
    date = ClosestDateParameter(default=datetime.date.today())
    prefix = luigi.Parameter(default="oai_dc")
    collection = luigi.Parameter(default="gallica:typedoc:partitions")

    def requires(self):
        return BNFMarcHarvest(begin=self.begin, end=self.date,
                              prefix=self.prefix, collection=self.collection)

    @timed
    def run(self):
        _, combined = tempfile.mkstemp(prefix='tasktree')
        for target in self.input():
            shellout("cat {input} >> {output}", input=target.path,
                     output=combined)
        output = shellout("marcuniq -o {output} {input}", input=combined)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='mrc', digest=True))

class BNFJson(BNFTask):
    """ Convert to JSON. """
    begin = luigi.DateParameter(default=datetime.date(2011, 1, 1))
    date = ClosestDateParameter(default=datetime.date.today())
    prefix = luigi.Parameter(default="oai_dc")
    collection = luigi.Parameter(default="gallica:typedoc:partitions")

    def requires(self):
        return BNFCombine(begin=self.begin, date=self.date, prefix=self.prefix,
                          collection=self.collection)

    def run(self):
        tmp = shellout("marctojson -m date={date} {input} > {output}",
                       input=self.input().path, date=self.date)
        luigi.File(tmp).move(self.output().path)

    def output(self):
        """ Use monthly files. """
        return luigi.LocalTarget(path=self.path(ext='ldj', digest=True))

class BNFIndex(BNFTask, CopyToIndex):
    begin = luigi.DateParameter(default=datetime.date(2011, 1, 1))
    date = ClosestDateParameter(default=datetime.date.today())
    prefix = luigi.Parameter(default="oai_dc")
    collection = luigi.Parameter(default="gallica:typedoc:partitions")

    index = 'bnf'
    doc_type = 'title'
    purge_existing_index = True

    mapping = {'title': {'date_detection': False,
                          '_id': {'path': 'content.001'},
                          '_all': {'enabled': True,
                                   'term_vector': 'with_positions_offsets',
                                   'store': True}}}

    def update_id(self):
        """ This id will be a unique identifier for this indexing task."""
        return self.effective_task_id()

    def requires(self):
        return BNFJson(begin=self.begin, date=self.date, prefix=self.prefix,
                       collection=self.collection)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301,C0103,W0614,W0401

"""
Various utilities.
"""

from __future__ import print_function
from gluish import colors
from luigi.task import Register, flatten
from siskin import __version__

import cStringIO as StringIO
import errno
import hashlib
import json
import luigi
import os
import pprint
import requests
import sys
import tempfile

MAN_HEADER = r"""

  \V/    \V/    \V/    \V/    \V/    \V/    \V/    \V/    \V/    \V/    \V/
---o------o------o------o------o------o------o------o------o------o------o-----

+ Siskin
"""

def generate_tasks_manual():
    """ Return a formatted listing of all tasks with their descriptions. """
    from siskin.sources import *
    from siskin.workflows import *

    output = StringIO.StringIO()
    # task_tuples = sorted(Register.get_reg().iteritems())
    task_names = Register.task_names()
    output.write(MAN_HEADER)
    output.write('  {0} tasks found\n\n'.format(len(task_names)))

    for name in task_names:
        klass = Register.get_task_cls(name)
        doc = klass.__doc__ or colors.red("@todo: docs")
        output.write('{0} {1}\n'.format(colors.green(name), doc))

        try:
            deps = flatten(klass().requires())
        except Exception:
            # TODO: tasks that have required arguments will fail here
            formatted = colors.yellow("\tUnavailable since task has required parameters.")
        else:
            formatted = '\t{0}'.format(pprint.pformat(deps).replace('\n', '\n\t'))
        output.write(colors.magenta('\n\tDependencies ({0}):\n\n{1}\n\n'.format(len(deps), formatted)))

    return output.getvalue()

def get_task_import_cache():
    """
    Load `taskname: modulename` mappings from dictionary. Return a tuple containing
    the dictionary and the path to the cache file.
    """
    task_import_cache = None
    path = os.path.join(tempfile.gettempdir(), 'siskin_task_import_cache_%s' % __version__)
    if not os.path.exists(path):
        from siskin.sources import *
        from siskin.workflows import *
        with open(path, 'w') as output:
            task_import_cache = dict([(name, Register.get_task_cls(name).__module__) for name in Register.task_names() if name[0].isupper()])
            json.dump(task_import_cache, output)

    if task_import_cache is None:
        with open(path) as handle:
            try:
                task_import_cache = json.load(handle)
            except Exception as err:
                print("failed load task import cache, try removing %s and then try again" % path, file=sys.stderr)
                sys.exit(1)

    return task_import_cache, path

class URLCache(object):
    """ A simple URL *content* cache. Stores everything on the filesystem.
    Content is first written to a temporary file and then renamed.
    With concurrent requests for the same URL, the last one wins.
    Raises exception on any statuscode >= 400. """

    def __init__(self, directory=None):
        self.directory = directory or tempfile.gettempdir()
        self.sess = requests.session()

    def get_cache_file(self, url):
        digest = hashlib.sha1(url).hexdigest()
        d0, d1, d2 = digest[:2], digest[2:4], digest[4:6]
        path = os.path.join(self.directory, d0, d1, d2)
        if not os.path.exists(path):
            try:
                os.makedirs(path)
            except OSError as e:
                if e.errno == errno.EEXIST:
                    pass
                else:
                    raise
        return os.path.join(path, digest)

    def is_cached(self, url):
        return os.path.exists(self.get_cache_file(url))

    def get(self, url):
        if not self.is_cached(url):
            r = self.sess.get(url)
            if r.status_code >= 400:
                raise RuntimeError('%s on %s' % (r.status_code, url))
            with tempfile.NamedTemporaryFile(delete=False) as output:
                output.write(r.text)
            os.rename(output.name, self.get_cache_file(url))

        with open(self.get_cache_file(url)) as handle:
            return handle.read()

class ElasticsearchMixin(luigi.Task):
    """ A small mixin for tasks that require an ES connection. """
    es_host = luigi.Parameter(default='localhost', significant=False,
                              description='elasticsearch host')
    es_port = luigi.IntParameter(default=9200, significant=False,
                                 description='elasticsearch port')



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301

from __future__ import print_function
from gluish.benchmark import timed
from gluish.common import Executable
from gluish.esindex import CopyToIndex
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.task import DefaultTask
import datetime
import luigi
import urllib

class RISMTask(DefaultTask):
    TAG = '014'

    def closest(self):
        return datetime.date(2014, 1, 1)

class RISMSync(RISMTask):
    """ Get at most n records (defaults to 20000). """
    date = ClosestDateParameter(default=datetime.date.today())
    maximum = luigi.IntParameter(default=20000, significant=False)

    def requires(self):
        return Executable(name='wget')

    @timed
    def run(self):
        base = 'http://service1.rism.info/opac'
        params = {
            'operation': 'searchRetrieve',
            'version': '1.1',
            'query': 'source=http*',
            'maximumRecords': self.maximum,
        }
        output = shellout("""wget --retry-connrefused -O {output} "{base}?{params}" """,
                          base=base, params=urllib.urlencode(params))
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml'))

class RISMMarc(RISMTask):
    """ Convert IMSLP XML (trans, antizebra) to MARCXML, then to MARC """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return {'file': RISMSync(date=self.date),
                'apps': [Executable(name='xsltproc'),
                         Executable(name='yaz-marcdump'),
                         Executable(name='marcuniq')]}

    @timed
    def run(self):
        # first trans.xsl
        output = shellout("xsltproc {stylesheet} {input} > {output}",
                          input=self.input().get('file').path,
                          stylesheet=self.assets('trans.xsl'))
        # then antizebra.xsl
        output = shellout("xsltproc {stylesheet} {input} > {output}",
                          input=output, stylesheet=self.assets('antizebra.xsl'))
        # then binary marc
        output = shellout("yaz-marcdump -i marcxml -o marc {input} > {output}",
                          input=output)
        # then marcuniq
        output = shellout("marcuniq -o {output} {input}", input=output)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='mrc'))

class RISMJson(RISMTask):
    """ Convert MARC to JSON. """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return RISMMarc(date=self.date)

    @timed
    def run(self):
        output = shellout("marctojson -m date={date} {input} > {output}",
                          input=self.input().path, date=self.closest())
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj'))

class RISMIndex(RISMTask, CopyToIndex):
    date = ClosestDateParameter(default=datetime.date.today())

    index = 'rism'
    doc_type = 'title'

    mapping = {'title': {'date_detection': False,
                          '_id': {'path': 'content.001'},
                          '_all': {'enabled': True,
                                   'term_vector': 'with_positions_offsets',
                                   'store': True}}}

    def update_id(self):
        """ This id will be a unique identifier for this indexing task."""
        return self.effective_task_id()

    def requires(self):
        return RISMJson(date=self.date)



#!/usr/bin/env python
# coding: utf-8

"""
siskin is a set of tasks for library metadata management.
~~~~~~
"""

import sys

if sys.version_info < (2, 7) or sys.version_info.major > 2:
    print("siskin runs with python 2.7 or higher, but not with python 3 yet")
    sys.exit(1)

try:
    from setuptools import setup
except:
    from distutils.core import setup

from siskin import __version__

install_requires = [
    'BeautifulSoup==3.2.1',
    'argparse>=1.2',
    'astroid==1.1.1',
    'colorama==0.3.3',
    'decorator==3.4.0',
    'elasticsearch==1.3.0',
    'gluish==0.1.79',
    'gspread==0.2.2',
    'jsonpath-rw==1.3.0',
    'logilab-common==0.61.0',
    'luigi==1.1.2',
    'lxml==3.4.2',
    'marcx==0.1.17',
    'nose==1.3.3',
    'ply==3.4',
    'prettytable==0.7.2',
    'protobuf==2.6.1',
    'pyisbn==1.0.0',
    'pylint==1.2.1',
    'pymarc==3.0.1',
    'pyparsing==2.0.3',
    'python-dateutil==2.2',
    'pytz==2014.4',
    'requests==2.5.1',
    'simplejson==3.6.0',
    'six==1.9.0',
    'snakebite==2.5.1',
    'sqlitebck==1.2.1',
    'urllib3==1.10',
    'wsgiref==0.1.2',
]

setup(name='siskin',
      version=__version__,
      description='Various sources and workflows.',
      url='https://github.com/miku/siskin',
      author='Martin Czygan',
      author_email='martin.czygan@gmail.com',
      packages=[
        'siskin',
        'siskin.sources',
        'siskin.workflows'
      ],
      package_dir={'siskin': 'siskin'},
      package_data={'siskin': ['assets/*']},
      scripts=[
        'bin/taskcat',
        'bin/taskcp',
        'bin/taskdeps-dot',
        'bin/taskdir',
        'bin/taskdo',
        'bin/taskdu',
        'bin/taskhead',
        'bin/taskhelp',
        'bin/taskhome',
        'bin/taskindex-delete',
        'bin/taskless',
        'bin/taskls',
        'bin/taskman',
        'bin/tasknames',
        'bin/taskopen',
        'bin/taskoutput',
        'bin/taskredo',
        'bin/taskrm',
        'bin/taskstatus',
        'bin/tasktree',
        'bin/taskversion',
        'bin/taskwc',
      ],
      install_requires=install_requires,
)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301

"""
Pilsen.

Configuration keys:

[pilsen]

src = [user@server:[port]]/path/to/directory
"""

from gluish.common import Directory
from gluish.format import TSV
from gluish.parameter import ClosestDateParameter
from gluish.path import iterfiles
from gluish.utils import shellout
from siskin.configuration import Config
from siskin.task import DefaultTask
import datetime
import luigi

config = Config.instance()

class PilsenTask(DefaultTask):
    TAG = '032'

class PilsenSync(PilsenTask):
    """ Just copy over the marc files. """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return Directory(path=self.taskdir())

    def run(self):
        shellout("rsync -avz {src} {dst}", src=config.get('pilsen', 'src'), dst=self.taskdir())
        with self.output().open('w') as output:
            for path in iterfiles(self.taskdir(), fun=lambda p: '-luigi-tmp-' not in p):
                output.write_tsv(path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='filelist'), format=TSV)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301

"""
Yago.
"""

from gluish.benchmark import timed
from gluish.intervals import yearly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.task import DefaultTask
import datetime
import luigi

class YagoTask(DefaultTask):
    TAG = 'yago'

    def closest(self):
        return yearly(self.date)

class YagoDump(YagoTask):
    """ Download and extract Yago dump. Might 404 from time to time. """
    date = ClosestDateParameter(default=datetime.date.today())

    @timed
    def run(self):
        url = "http://www.mpi-inf.mpg.de/yago-naga/yago/download/yago/yago2s_ttl.7z"
        output = shellout("""wget --retry-connrefused -O {output} {url}""", url=url)
        output = shellout("""7z e -so {input} 2> /dev/null > {output}""", input=output)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ttl'))



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301

"""
NAXOS.

Configuration keys:

[naxos]

ftp-host = host.name
ftp-username = username
ftp-password = password
ftp-filepath = /path/to/marc-nml-gesamt.zip
"""

from __future__ import print_function
from gluish.benchmark import timed
from gluish.common import FTPFile
from gluish.esindex import CopyToIndex
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.configuration import Config
from siskin.task import DefaultTask
import datetime
import luigi

config = Config.instance()

class NaxosTask(DefaultTask):
    TAG = '005'

    def closest(self):
        return datetime.date(2014, 1, 1)

class NaxosJson(NaxosTask):
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return FTPFile(host=config.get('naxos', 'ftp-host'),
                       username=config.get('naxos', 'ftp-username'),
                       password=config.get('naxos', 'ftp-password'),
                       filepath=config.get('naxos', 'ftp-filepath'))

    @timed
    def run(self):
        """ Unzip and strip namespace. """
        output = shellout("""unzip -p {input} {path} |
                             LANG=C perl -lnpe 's@xmlns:marc="http://www.loc.gov/MARC21/slim"@@g' |
                             LANG=C perl -lnpe 's@<marc:@<@' |
                             LANG=C perl -lnpe 's@</marc:@</@' > {output}""",
                          input=self.input().path, path='marc-nml-gesamt.xml')
        output = shellout("marcxmltojson -m date={date} {input} > {output}", date=self.closest(), input=output)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj'))

class NaxosIndex(NaxosTask, CopyToIndex):
    date = ClosestDateParameter(default=datetime.date.today())

    index = 'naxos'
    doc_type = 'title'
    purge_existing_index = True

    settings = {
        "settings": {
            "number_of_shards": 5,
            "analysis": {
                "filter": {
                    "autocomplete_filter": {
                        "type":     "edge_ngram",
                        "min_gram": 1,
                        "max_gram": 20
                    }
                },
                "analyzer": {
                    "autocomplete": {
                        "type":      "custom",
                        "tokenizer": "standard",
                        "filter": [
                            "lowercase",
                            "autocomplete_filter"
                        ]
                    }
                }
            }
        }
    }

    mapping = {
        'title': {
            'date_detection': False,
            '_id': {
                'path': 'content.001'
            },
            '_all': {
                'enabled': True,
                'term_vector': 'with_positions_offsets',
                'store': True
            },
            'properties': {
                'content': {
                    'properties': {
                        '245': {
                            'properties': {
                                'a': {
                                    'type': 'string',
                                    'analyzer': 'autocomplete'
                                }
                            }
                        },
                        '100': {
                            'properties': {
                                'a': {
                                    'type': 'string',
                                    'analyzer': 'autocomplete'
                                }
                            }
                        },
                        '700': {
                            'properties': {
                                'a': {
                                    'type': 'string',
                                    'analyzer': 'autocomplete'
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    def update_id(self):
        """ This id will be a unique identifier for this indexing task."""
        return self.effective_task_id()

    def requires(self):
        return NaxosJson(date=self.date)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103

"""
Hackernews stories and comments.
"""

from gluish.utils import shellout
from siskin.task import DefaultTask
import luigi

class HNTask(DefaultTask):
    TAG = 'hackernews'

class HNCommentsDownload(HNTask):
    """ Download comments. """

    def run(self):
	url = "https://ia902503.us.archive.org/33/items/HackerNewsStoriesAndCommentsDump/HNCommentsAll.7z"
	output = shellout("wget -O {output} {url}", url=url)
	luigi.File(output).move(self.output().path)

    def output(self):
	return luigi.LocalTarget(path=self.path(ext='7z'))



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101

"""
HS ZI/GR

Configuration keys:

scp-src = [user@server:[port]]/path/to/file/{tag}/*mrc
"""

from gluish.benchmark import timed
from gluish.esindex import CopyToIndex
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.configuration import Config
from siskin.task import DefaultTask
import datetime
import luigi

config = Config.instance()

class HSZIGRTask(DefaultTask):
    TAG = '023'

    def closest(self):
        """ One time thing. """
        return datetime.date(2014, 1, 1)

class HSZIGRImport(HSZIGRTask):
    date = ClosestDateParameter(default=datetime.date.today())

    @timed
    def run(self):
        output = shellout("scp {origin} {output}", origin=config.get('hszigr', 'scp-src'))
        output = shellout("marcuniq -x 'BW, ZI' {input} > {output}", input=output)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='mrc'))

class HSZIGRJson(HSZIGRTask):
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return HSZIGRImport(date=self.date)

    @timed
    def run(self):
        output = shellout("marctojson -m date={date} {input} > {output}",
                          input=self.input().path, date=self.closest())
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj'))

class HSZIGRIndex(HSZIGRTask, CopyToIndex):
    date = ClosestDateParameter(default=datetime.date.today())

    index = 'hszigr'
    doc_type = 'title'
    pruge_existing_index = True

    mapping = {'title': {'date_detection': False,
                      '_id': {'path': 'content.001'},
                      '_all': {'enabled': True,
                               'term_vector': 'with_positions_offsets',
                               'store': True}}}
    def update_id(self):
        return self.effective_task_id()

    def requires(self):
        return HSZIGRJson(date=self.date)



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,E1103,C0301

from gluish.benchmark import timed
from gluish.common import OAIHarvestChunk, Executable
from gluish.esindex import CopyToIndex
from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.utils import date_range, shellout
from siskin.task import DefaultTask
import datetime
import luigi
import tempfile

class DissonTask(DefaultTask):
    TAG = '013'

    def closest(self):
        return monthly(date=self.date)

class DissonHarvest(luigi.WrapperTask, DissonTask):
    """ Harvest Disson. """

    begin = luigi.DateParameter(default=datetime.date(2000, 1, 1))
    end = ClosestDateParameter(default=datetime.date.today())
    prefix = luigi.Parameter(default="oai_dc", significant=False)
    url = luigi.Parameter(default="http://services.dnb.de/oai/repository", significant=False)
    collection = luigi.Parameter(default="dnb:online:dissertations", significant=False)

    def requires(self):
        """ Harvest in 1-month chunks. """
        if self.end < self.begin:
            raise RuntimeError('Invalid range: %s - %s' % (self.begin, self.end))
        dates = date_range(self.begin, self.end, 1, 'months')
        for begin, end in zip(dates[:-1], dates[1:]):
            yield OAIHarvestChunk(begin=begin, end=end, prefix=self.prefix,
                                  url=self.url, collection=self.collection)

    def output(self):
        return self.input()

class DissonCombine(DissonTask):
    """ Combine the chunks for a date range into a single file.
    The filename will carry a name, that will only include year
    and month for begin and end, example:

    .../013/disson-combine/2010-01--2013-11.mrc"""

    begin = luigi.DateParameter(default=datetime.date(2000, 1, 1))
    date = ClosestDateParameter(default=datetime.date.today())
    prefix = luigi.Parameter(default="oai_dc", significant=False)
    url = luigi.Parameter(default="http://services.dnb.de/oai/repository", significant=False)
    collection = luigi.Parameter(default="dnb:online:dissertations", significant=False)

    def requires(self):
        return {'files': DissonHarvest(begin=self.begin, end=self.date,
                                       prefix=self.prefix, url=self.url,
                                       collection=self.collection),
                'apps': [Executable(name='xsltproc'),
                         Executable(name='yaz-marcdump'),
                         Executable(name='marcuniq')]}

    @timed
    def run(self):
        xsl = self.assets("013_OAIDCtoMARCXML.xsl")
        _, combined = tempfile.mkstemp(prefix='siskin-')
        for target in self.input().get('files'):
            output = shellout("xsltproc {stylesheet} {input} > {output}",
                              stylesheet=xsl, input=target.path)
            output = shellout("yaz-marcdump -i marcxml -o marc {input} > {output}",
                              input=output)
            shellout("cat {input} >> {output}", input=output, output=combined)

        output = shellout("marcuniq -o {output} {input}", input=combined)
        luigi.File(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='mrc'))

class DissonJson(DissonTask):
    """ Convert to JSON. """

    begin = luigi.DateParameter(default=datetime.date(2000, 1, 1))
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return DissonCombine(begin=self.begin, date=self.date)

    def run(self):
        tmp = shellout("marctojson -m date={date} {input} > {output}",
                       input=self.input().path, date=self.date)
        luigi.File(tmp).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj'))

class DissonIndex(DissonTask, CopyToIndex):
    begin = luigi.DateParameter(default=datetime.date(2000, 1, 1))
    date = ClosestDateParameter(default=datetime.date.today())

    index = "disson"
    doc_type = "title"
    purge_existing_index = True

    mapping = {'title': {'date_detection': False,
                          '_id': {'path': 'content.001'},
                          '_all': {'enabled': True,
                                   'term_vector': 'with_positions_offsets',
                                   'store': True}}}

    def update_id(self):
        """ This id will be a unique identifier for this indexing task."""
        return self.effective_task_id()

    def requires(self):
        return DissonJson(begin=self.begin, date=self.date)
